# $Id: exception.py 2136 2006-09-08 19:46:55Z dairiki $
# exception.py - exception classes for Myghty
# Copyright (C) 2004, 2005 Michael Bayer mike_mp@zzzcomputing.com
# Original Perl code and documentation copyright (c) 1998-2003 by Jonathan Swartz. 
#
# This module is part of Myghty and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php
# 

import string, sys, re, traceback, os
from myghty.util import *

# get the directory of myghty, to filter it out of friendly stack traces
import myghty
MYGHTY_DIR = os.path.normcase(os.path.dirname(myghty.__file__))
        
class Error(Exception):
    """generic exception class"""
    
    def __init__(self, message=None, wrapped=None, *args, **params):
        self.wrapped = wrapped

        if message is None:
            if self.wrapped is None:
                self.msg = self.__class__.__name__
            else:
                self.msg = "%s(%s): %s" % (
                    self.__class__.__name__,
                    self.wrapped.__class__.__name__,
                    self.wrapped)
        else:
            self.msg = self.__class__.__name__ + ": " + message

        self.trace_records = []
        self.lead_rec = None
        
        self.reversefiles = {}

        if self.wrapped is not None and isinstance(self.wrapped, SyntaxError):
            self.lead_rec = Error.TBTraceLine(Error.SimpleTraceLine(self.wrapped.text, self.wrapped.lineno, self.wrapped.filename))
            self.ispythonsyntax = True
        else:
            self.ispythonsyntax = False
            
        Exception.__init__(self, *args, **params)
        
        
    def initTraceback(self, interpreter = None):
        """ call this right before each re-raise of the exception"""

        # at the moment, initTraceback only needs to be called once, so
        # return if we already got trace records
        if len(self.trace_records): return

        # extract stack info from the exception throw:
        (type, value, trcback) = sys.exc_info()
        self.raw_excinfo = (type, value, trcback)
        rawrecords = traceback.extract_tb(trcback)

        # add to that, stack info from the current stack frame
        rawrecords = traceback.extract_stack() + rawrecords

        # if a lead record was programmatically hardcoded, init that
        # (it is probably a SyntaxTraceLine from Lexer)
        # and dont change it
        if self.lead_rec is not None:
            self.lead_rec.init_reverse_file(self, interpreter)
            set_lead_rec = False
        else:
            set_lead_rec = True
            
        recs = []
        
        # create TBTraceLines out of the stack frames in an attempt to sort out
        # compiled templatelines, myghty library lines, and non-myghty python lines
        for rec in rawrecords:
            rec = Error.TBTraceLine(Error.SimpleTraceLine(rec[3], rec[1], rec[0]))
            rec.init_reverse_file(self, interpreter)
            
            if not rec.is_myghty_lib() and set_lead_rec:
                self.lead_rec = rec
            recs.insert(0, rec)
        
        self.trace_records += recs

    class CodeLine:
        """represents a line of code in a python source file"""
        def __init__(self, linenum, line, encoding=None):
            self.linenum = linenum
            if line is None:
                self.line = 'None'
            else:
                self.line = _EncodedLine(string.rstrip(line), encoding)

    class TraceLine:
        """represents a trace line in a stack trace"""
        
        def __init__(self):
            self.codeline = None
            self.code = None
            self.file = None
            self.original = None
            
        def init_reverse_file(self, parent, interpreter):raise NotImplementedError()



        def is_myghty_lib(self):
            "returns True if this line corresponds to package in the myghty distribution"
            return self.has_file() and os.path.commonprefix([MYGHTY_DIR, os.path.normcase(self.file)]) == MYGHTY_DIR
            

        def get_lines(self, span = 3):raise NotImplementedError()
        
        def get_file_lines(self, f, linenum, span):
            i = 0
            lines = []
            encoding = self.get_source_encoding()

            while True:
                l = f.readline()
                i += 1
                if i in range(linenum - span, linenum + span + 1):
                    lines.append(Error.CodeLine(i, l, encoding))
                if i == linenum + span: break

            f.close()
            return lines
        
        def has_file(self):
            return self.file is not None and os.access(self.file, os.F_OK)

        def get_source_encoding(self):
            raise NotImplementedError

        
        
    class SyntaxTraceLine(TraceLine):
        """a traceline generated by Lexer to mark a pre-python syntax error"""
        
        def __init__(self, code, line, name, source, file,
                     source_encoding=None):
            self.code = code
            self.line = line
            self.name = name
            self.file = file
            self.source = source
            self.init = False
            self.original = self
            self.codeline = self.line
            if source_encoding is None:
                source_encoding = sys.getdefaultencoding()
            self.source_encoding = source_encoding

        def init_reverse_file(self, parent, interpreter):
            pass

        def get_lines(self, span = 3):
            f = StringIO(self.source)
            return self.get_file_lines(f, self.line, span)

        def get_source_encoding(self):
            return self.source_encoding
        
    class SimpleTraceLine(TraceLine):
        """a traceline for a regular python file"""
        
        def __init__(self, code, line, file):
            self.__code = code
            self.line = line
            self.codeline = self.line
            self.original = self
            self.file = file
            self.compiled_record = None
            
        def init_reverse_file(self, parent, interpreter):
            pass
            
        def _get_code(self):
            if self.__code is None:
                return self.get_lines(span = 0)[0].line
            else:
                return self.__code
        code = property(_get_code)      

        def get_lines(self, span = 3):
            if self.compiled_record is not None:
                f = self.compiled_record.get_compiled_source()
            elif not self.has_file():
                return [Error.CodeLine(self.line, self.__code)] #FIXME: encoding
            else:
                f = file(self.file)

            return self.get_file_lines(f, self.line, span)

        def get_source_encoding(self):
            if self.compiled_record is not None:
                f = self.compiled_record.get_compiled_source()
            elif not self.has_file():
                return sys.getdefaultencoding() # FIXME: this is wrong
            else:
                f = file(self.file)
            return _parse_encoding(f)

    class TBTraceLine(TraceLine):
        """a wrapper for a simpletraceline that attempts to map its python string or file to an 
        originating myghty template string or file, with line numbers cross-linked between them."""
        
        def __init__(self, original):
            self.original = original
            
        def init_reverse_file(self, parent, interpreter):
            original = self.original
            file = original.file
            self.ismyghtylib = None
                
            if parent.reversefiles.has_key(file):
                self.reversefile = parent.reversefiles[file]
            elif interpreter is not None and interpreter.reverse_lookup.has_key(file):
                comprec = interpreter.reverse_lookup[file]
                self.reversefile = Error.ReverseFile(interpreter, compiled_record = comprec)
                parent.reversefiles[file] = self.reversefile
            else:
                self.reversefile = None

            if self.reversefile is not None:
                self.codeline = self.reversefile.get_line_number(original.codeline)
                self.original.compiled_record = self.reversefile.compiled_record
                if self.codeline is None:
                    # if we are a compiled template and the line number is within the
                    # area beyond user-defined code, then we are a myghtylib traceline 
                    self.ismyghtylib = True
            else:
                self.codeline = None

            if self.codeline is None:
                self.codeline = original.codeline
                self.reversefile = None


        def is_myghty_lib(self):
            if self.ismyghtylib is not None:
                return self.ismyghtylib
            else:
                return Error.TraceLine.is_myghty_lib(self)

            
        def _get_code(self):
            if self.reversefile is not None:
                return self.get_lines(span = 0)[0].line
            else:
                return self.original.code
                
        code = property(_get_code, lambda s,p:None)
        
        def _get_file(self):
            if (
                self.reversefile is not None
                and 
                self.reversefile.compiled_record.csource.file_path is not None):
                
                return self.reversefile.compiled_record.csource.file_path
            else:
                return self.original.file
                
        file = property(_get_file, lambda s,p:None)

        def get_lines(self, span = 3):
            if self.reversefile is not None:
                return self.get_file_lines(self.reversefile.get_source(), self.codeline, span)
            else:
                #return [Error.CodeLine(self.codeline, self.original.get_code())]
                return self.original.get_lines(span)

        def get_source_encoding(self):
            return self.original.get_source_encoding()
        
    class ReverseFile:
        """represents a python source string or file, a pointer back to
        the myghty template file or string that originated it, and a cross reference of
        all line numbers in the python file that map back to the template file."""
    
        def __init__(self, interpreter, compiled_record):
            self.sourcemap = []
            self.compiled_record = compiled_record
            self.isvalid = False

            f = compiled_record.get_compiled_source()                   
            try:
                line = f.readline()
                match = re.match(r"# File: (\S+) CompilerID: (.*?) Timestamp: (.*?)", line)
                if not match: return

                (self.path, self.compilerid, self.timestamp) =  \
                (match.group(1), match.group(2), match.group(3) )

                self.sourcemap.append(None)
                state = 0
                linecounter = 0

                for line in f:
                    if state == 0:
                        self.sourcemap.append(None)
                        match = re.match(r"\s*#\s*BEGIN BLOCK (\w+)", line)
                        if match:
                            state = 1

                        continue

                    elif state == 1:
                        match = re.match(r"\s*#\s*BEGIN CODE BLOCK", line)
                        if match:
                            state = 2
                            self.sourcemap.append(None)
                            continue
                    elif state == 2:
                        match = re.match(r"\s*#\s*END CODE BLOCK", line)
                        if match:
                            state = 1
                            self.sourcemap.append(None)
                            continue
                    
                        
                    if state == 1 or state == 2:    
                        match = re.match(r"\s*#\s*SOURCE LINE (\w+)", line)
                        if match:
                            self.sourcemap.append(None)
                            linecounter = int(match.group(1))   
                        else:
                            match = re.match(r"\s*#\s*END BLOCK (\w+)", line)
                            if match:
                                self.sourcemap.append(None)
                                state = 0
                            else:
                                self.sourcemap.append(linecounter)
                                if state == 2: linecounter += 1

                self.isvalid = True
            finally:
                f.close()
                
        def get_source(self):
            try:
                return self.compiled_record.csource.get_component_source_file()
            except:
                raise "Cant open file '%s'" % self.compiled_record.csource.file_path
            
        def get_line_number(self, linenum):
            if linenum < len(self.sourcemap):
                return self.sourcemap[linenum - 1]
            else:
                return None


    def htmlformat(self):
        return HTMLErrorFormatter(self).format()

    def textformat(self):
        return PlainTextErrorFormatter(self).format()

    def format(self):
        return PlainTextErrorFormatter(self).format()
        
    def singlelineformat(self):
        if len(self.trace_records):
            return "%s at %s line %d" % (self.msg, self.file, self.codeline)
        else:
            return self.msg
            
    file =property(lambda self: self.trace_records[0].file)
    

    codeline = property(lambda self:self.trace_records[0].codeline)
    
        
            
    def __str__(self):
        return self.singlelineformat()

    def is_python_syntax(self):
        return self.ispythonsyntax
        
    def message(self):
        return self.msg


class ErrorFormatter:
    """base class for an object that returns a string representation of an error"""
    
    def format(self):
        try:
            return self.do_format()
        except Exception, e:
            return "ErrorFormatter had an exception: " + str(e)
        except:
            e = sys.exc_info()[0]
            return "ErrorFormatter had an exception: " + str(e)


class PlainTextErrorFormatter(ErrorFormatter):
    def __init__(self, error):
        self.error = error
        error.initTraceback(None)
        self.records = self.error.trace_records
        self.lead_rec = self.error.lead_rec

    def do_format(self):
        templatetrace = self._format_traceback(self.records, True)
    
        realtrace = self._format_traceback(self.records, False)

        if self.lead_rec is not None:
            ret = (self._format_record(self.error.message(), self.lead_rec, templatetrace) +
                "\n---------------------------------------------\n" +
                               "Original Stack Trace:\n");
                               
            if self.error.is_python_syntax():
                ret += self._format_record(self.error.message(), self.lead_rec.original, realtrace)
            else:
                ret += self._format_record(self.error.message(), self.records[0].original, realtrace)
                
            return ret
        else:
            return self._format_record(self.error.message(), self.records[0], realtrace)

    def _format_traceback(self, records, friendly):
        trace = ""
        for rec in self.records:
            if not friendly:
                rec = rec.original

            if not friendly or not rec.is_myghty_lib():
                trace += "%s:%s\n" % (rec.file, rec.codeline)

        return trace

    def _format_record(self, message, rec, trace):
        frec = (
            message + "\n\n" + 
            "file: %s line %s\n" % (rec.file, rec.codeline)
            )
    
        frec += "\n" + str(rec.code);
        if trace: frec += "\n\n" + trace

        return frec

class HTMLErrorFormatter(ErrorFormatter):
    def do_format(self):
        buffer = StringIO()

        try:            
            self.interp.execute(self.comp,
                                out_buffer=buffer,
                                output_encoding='latin1', # Default for HTML
                                encoding_errors='htmlentityreplace',
                                request_args = argdict(
                records = self.records,
                lead_rec = self.lead_rec,
                error = self.error
            ))
            return buffer.getvalue()
        except Exception, e:
            nestederror = Error(wrapped = e)
        except:
            nestederror = Error(wrapped = sys.exc_info()[0])

        nestederror.initTraceback(self.interp)
        return ("<pre>%s\n\nAdditionally, HTMLErrorFormatter had the following error:\n\n%s</pre>" % 
            (PlainTextErrorFormatter(self.error).format(),
            PlainTextErrorFormatter(nestederror).format()))


    def __init__(self, error):
        import myghty.interp as interp
        self.error = error
        error.initTraceback(None)
        self.records = self.error.trace_records
        self.lead_rec = self.error.lead_rec

        def handle(e, m, **params):
            raise e

        self.interp = interp.Interpreter(error_handler = handle)
        self.comp = self.interp.make_component("""
<%args>
    records
    lead_rec
    error
</%args>
<html>
<head>
    <title>Myghty Template Error</title>
    <style>
        body, td {
            margin: 20px;
            font-family: verdana, sans-serif;
            font-size: 8pt;
        }
        .red {
            color:#FF0000;
        }
        .bold {
            font-weight: bold;
        }
    </style>
</head>
<body>

% if lead_rec is not None:
    <h2>Myghty Template Error</h2>

    <& formatrecord, message = error.message(), lead_rec = lead_rec, records = records, friendly = True &>

    <br/>
    <hr/>
    <h3>Original Stack Trace:</h3>
%   if error.is_python_syntax():
        <& formatrecord, message = error.message(), lead_rec = lead_rec.original, records = records, friendly = False &>
%   else:
        <& formatrecord, message = error.message(), lead_rec = records[0].original, records = records, friendly = False &>
%
% else:
    <& formatrecord, message = error.message(), lead_rec = records[0], records = records, friendly = False &>
%   
</body>
</html>

<%def formatrecord>
<%args>
    message
    lead_rec
    records
    friendly
</%args>

    <table>
    <tr><td class="bold">Error:</td>
    <td><% message |h %></td>
    <tr><td class="bold">File:</td>
    <td><% lead_rec.file %> line <% lead_rec.codeline %></td>
    </tr>
    <tr>
    <td class="bold" valign="top">Context:</td>
    <td>
    <& formatlines, lines = lead_rec.get_lines(), highlight = lead_rec.codeline &>
    </td>
    </tr>
    
    <td class="bold" valign="top">Traceback:</td>
    <td>
    <& formattraceback, records = records, friendly = friendly &>
    </td>
    </tr>
    </table>
</%def>

<%def formatlines>
<%args>
lines
highlight
</%args>

% for line in lines:
%   code = m.apply_escapes( line.line.expandtabs(), 'h' )
%   code = code.replace(' ','&nbsp;')
%   if line.linenum == highlight:
    <span class="red"><% line.linenum %>: <% code %></span><br/>
%   else:
    <% line.linenum %>: <% code %><br/>
%
</%def>

<%def formattraceback>
<%args>
    records
    friendly
</%args>

% for rec in records:
%   if not friendly:
%       rec = rec.original

%   if not friendly or not rec.is_myghty_lib():
        <% rec.file %>:<% rec.codeline %><br/>  
%

</%def>
""", id='exception')

    
class Syntax(Error):
    "invalid syntax in a component"
    def __init__(self, error, comp_name, source_line, line_number,
                 source = None,
                 file = None,
                 source_encoding = None):
        self.error = error
        Error.__init__(self, error)
        self.lead_rec = Error.SyntaxTraceLine(source_line, line_number,
                                              comp_name,
                                              source, file, source_encoding)

class AbortRequest(Exception):
    "request aborted"
    pass
    
class Abort(AbortRequest):
    "HTTP abort called by component"
    def __init__(self, aborted_value, reason):
        self.aborted_value = aborted_value
        self.reason = reason

class Redirected(Abort):
    "redirect called"
    def __init__(self, path, code=301, reason=None):
        self.path = path
        Abort.__init__(self, code, reason)

            
class Decline(AbortRequest):
    "decline called by component"
    def __init__(self, declined_value):
        self.declined_value = declined_value

class ConfigurationError(Error):
    "configuration error"
    pass

class ServerError(Error):
    "server error (500) "
    pass

class Request(Error):
    "request error"
    pass
            
class Compiler(Error):
    "compiler error"
    pass

class Interpreter(Error):
    "interpreter error"
    pass

class MethodNotFound(Error):
    "couldnt find method"
    pass
        
class ComponentNotFound(Error):
    "couldnt find component"
    def __init__(self, msg, resolution_detail, silent = False):
        Error.__init__(self, msg)
        self.resolution_detail = resolution_detail
        self.silent = silent
        
    def message(self):
        if self.resolution_detail is not None:
            return self.msg + " (" + string.join(self.resolution_detail, ', ') + ")"
        else:
            return self.msg

    def create_toplevel(self):
        """converts this ComponentNotFound into a TopLevelNotFound exception, which 
        is used by HTTPHandlers to return a 404 return code."""
        return TopLevelNotFound(self.msg, self.resolution_detail, self.silent)
        
class PathIsDirectory(ComponentNotFound):
    "the specified component path is a directory"
    pass

class TopLevelNotFound(ComponentNotFound):
    "couldnt find top level component"
    pass

class MissingArgument(Error):
    "a required argument was missing from a component call"
    pass
    
class IncompatibleCompiler(Error):
    "wrong compiler"
    pass

################################################################

import codecs, parser

# Regexp to match python magic encoding line
_PYTHON_MAGIC_COMMENT_re = re.compile(
    r'[ \t\f]* \# .* coding[=:][ \t]*([-\w.]+)',
    re.VERBOSE)

def _parse_encoding(fp): 
    """Deduce the encoding of a source file from magic comment. 
	 
    It does this in the same way as the `Python interpreter`__ 
	 
    .. __: http://docs.python.org/ref/encodings.html 
	 
    The ``fp`` argument should be a seekable file object.
    """
    pos = fp.tell()
    fp.seek(0)
    try:
        line1 = fp.readline()
        has_bom = line1.startswith(codecs.BOM_UTF8)
        if has_bom:
            line1 = line1[len(codecs.BOM_UTF8):]

        m = _PYTHON_MAGIC_COMMENT_re.match(line1)
        if not m:
            try:
                parser.suite(line1)
            except SyntaxError:
                # Either it's a real syntax error, in which case the source
                # is not valid python source, or line2 is a continuation of
                # line1, in which case we don't want to scan line2 for a magic
                # comment.
                pass
            else:
                line2 = fp.readline()
                m = _PYTHON_MAGIC_COMMENT_re.match(line2)

        if has_bom:
            if m:
                raise SyntaxError, \
                      "python refuses to compile code with both a UTF8" \
                      " byte-order-mark and a magic encoding comment"
            return 'utf_8'
        elif m:
            return m.group(1)
        else:
            return None
    finally:
        fp.seek(pos)
                

class _EncodedLine(object):
    def __init__(self, line, encoding=None):
        if encoding is None:
            encoding = sys.getdefaultencoding()
        self.line = line
        self.encoding = encoding

    def __str__(self):
        if isinstance(self.line, str):
            return self.line
        return self.line.encode('ascii', 'replace')

    def __unicode__(self):
        if isinstance(self.line, unicode):
            return self.line
        return self.line.decode(self.encoding, 'replace')

    def expandtabs(self, tabsize=8):
        return _EncodedLine(self.line.expandtabs(tabsize), self.encoding)
    



import memcache
from myghty.synchronization import *
from myghty.container import NamespaceManager, Container

import sys

class MemcachedNamespaceManager(NamespaceManager):

    def __init__(self, context, namespace, url, **params):
        NamespaceManager.__init__(self, context, namespace, **params)
        self.mc = memcache.Client([url], debug=0)

    # memcached does its own locking.  override our own stuff
    def do_acquire_read_lock(self): pass
    def do_release_read_lock(self): pass
    def do_acquire_write_lock(self, wait = True): return True
    def do_release_write_lock(self): pass

    # override open/close to do nothing, keep memcache connection open as long
    # as possible
    def open(self, *args, **params):pass
    def close(self, *args, **params):pass

    def __getitem__(self, key):
        value = self.mc.get(self.namespace + "_" + key)
        if value is None:
            raise KeyError(key)
        return value

    def __contains__(self, key):
        return self.mc.get(self.namespace + "_" + key) is not None

    def has_key(self, key):
        return self.mc.get(self.namespace + "_" + key) is not None

    def __setitem__(self, key, value):
        keys = self.mc.get(self.namespace + ':keys')
        if keys is None:
            keys = {}
        keys[key] = True
        self.mc.set(self.namespace + ':keys', keys)
        self.mc.set(self.namespace + "_" + key, value)

    def __delitem__(self, key):
        keys = self.mc.get(self.namespace + ':keys')
        try:
            del keys[key]
            self.mc.delete(self.namespace + "_" + key)
            self.mc.set(self.namespace + ':keys', keys)
        except KeyError:
            raise

    def do_remove(self):
        pass

    def keys(self):
        keys = self.mc.get(self.namespace + ':keys')
        if keys is None:
            return []
        else:
            return keys.keys()

class MemcachedContainer(Container):

    def do_init(self, **params):
        self.funclock = None

    def do_create_namespace_manager(self, context, namespace, url, **params):
        return MemcachedNamespaceManager(context, namespace, url, **params)

    def lock_createfunc(self, wait = True):
        if self.funclock is None:
            self.funclock = Synchronizer(identifier =
"memcachedcontainer/funclock/%s" % self.namespacemanager.namespace,
use_files = True, lock_dir = self.namespacemanager.lock_dir)

        return self.funclock.acquire_write_lock(wait)

    def unlock_createfunc(self):
        self.funclock.release_write_lock()




import os, sys
from myghty.resolver import *

publish_dir = os.path.dirname(os.path.abspath(__file__))

sys.path.extend((publish_dir + '/lib',) )

# The following options are passed directly into Myghty, so all configuration options
# available to the Myghty handler are available for your use here
config = {}


config['data_dir'] = os.path.join(publish_dir, 'cache')

config['component_root'] = [
    {'templates' : os.path.join(publish_dir, 'templates')},
    {'comp' : os.path.join(publish_dir, 'components')},
]


config['use_session'] = True
config['resolver_strategy'] = [

              # request-level resolution
              ConditionalGroup(context='request', rules=[

                      # bounce anything that is not "/" or ".myt" down to silent "not found"
                      # controller, which will forward out of Myghty to the Paste file handler
                      ConditionalGroup(regexp=r'(?!.*(/|\.myt)$)', rules = [
                              NotFound(silent=True)
                      ]),

                      # everything else request-level is handled by the controller.
                      # here we are using ResolvePathModule to automatically match pathnames
                      # to methods on objects inside the controller.  Other options include
                      # the more explicit ResolveModule as well as the new Routes resolver.
                      ResolvePathModule('controller'),

                      # ...or not found.
                      NotFound(),
              ]),

              # these rules then handle all other component calls,
              # including subrequest, component, and inherited
              PathTranslate(),
              ResolveDhandler(),
              URICache(),
              ResolveUpwards(),
              ResolvePathModule(),
              ResolveModule(),
              ResolveFile()
          ]



import myghty.interp as interp
import myghty.buffer as buffer
import myghty.exception as exception
import os, sys, gc, random, StringIO, time, thread, traceback
import testbase
import unittest


use_static_source = False

try:
    import threadframe
    has_threadframe = True
except:
    has_threadframe = False

# creates file based components with unique names, executes them, then deletes them.
# watch the memory usage to see if it creeps up.  with the 
# advent of the FileComponent __del__ method 
# cleaning its module from sys.modules, it should stay constant.  


threadcount = 0
gtime = time.time()
ccount = 240

use_files = True

def doit(i, regen):
    filename = "comp%s.myt" % i

    resolution = interpreter.resolve_component(filename)
    csource = resolution.csource
    
    if (regen > 6):
        # force a reload now and then
        csource.last_modified = time.time()
        pass
        
    comp = interpreter.load_component(csource)
    outbuf = StringIO.StringIO()
    m = interpreter.make_request(comp, out_buffer = outbuf)

    interpreter.debug("Executing.... %s id %d num elements in cache is %d  sys.modules size is %d " % (
        filename,
        id(comp),
        len(interpreter.code_cache.dict),
        len(sys.modules)
    ))
    m.execute()
    interpreter.debug("Done executing %s" % filename)

    comp = None
        


def do_test():
    global threadcount
    global gtime
    
    threadcount += 1
    # now randomly recreate files and execute the components in them
    try:
        for i in range(1,1000):
            doit(random.randint(1, ccount), random.randint(1,8))
            gtime = time.time()
            time.sleep(.000005)

    except exception.Error, e:
        print "thread id %s" % thread.get_ident()
        e.initTraceback(interpreter)
        print e.textformat()
    except Exception, e2:
        import traceback
        traceback.print_exc(file=sys.stdout)

    print "THREAD EXITING !!!! threadcount is %d" % threadcount
    threadcount -= 1


    
class TestMemUsage(testbase.MyghtyTest):
    def setUp(self):
        # now make (ccount) myghty files
        for i in range(1,ccount + 1):
            self.makefile(i)

        print "created %d files" % ccount
        
        # make an interpreter, with a really freekin small cache
        global interpreter
        if use_files:
            data_dir = self.cache
        else:
            data_dir = None

        interpreter = interp.Interpreter(delete_modules = True, component_root = self.htdocs, data_dir=data_dir, code_cache_size=5000, raise_error = True, debug_file=buffer.LinePrinter(sys.stdout), debug_threads = True, debug_elements=['codecache'], use_static_source = use_static_source)



    def makefile(self, i):
        filename = "comp%s.myt" % i
        c = """
    <%global>
        import os
    </%global>
        i am component #%s\n" % i
        
        # this kind of call breaks if sys.modules isnt populated with
        # this component's module
        os info is: <% os.getcwd() %>
    """
        self.create_file(self.htdocs, filename, c)

    def delete(self, i):    
        filename = "comp%s.myt" % i
    
        resolution = interpreter.resolve_component(filename)
        csource = resolution.csource
        if use_files:
            object_files = \
                    interpreter.get_component_object_files(csource)
    
            #print "deleting %s, %s" % (filename, repr(object_files))
            for f in object_files:
                try:
                    os.unlink(f)
                except OSError, e:
                    # (SIC): print this?  ignore this?
                    "Error deleting %s, probably never called" % f
    
        self.remove_file(self.htdocs, filename)        

    def testmemory(self):
	print "starting memory test"

        # now..threads !
        for t in range(1, 10):
            thread.start_new_thread(do_test, ())

        time.sleep(5)
        while threadcount > 0:
            time.sleep(1)    
            if time.time() - gtime > 10:
                print "deadlock !"
                if has_threadframe:
                    frames = threadframe.dict()
                    for thread_id, frame in frames.iteritems():
                        print '-' * 72
                        print '[%s] %d' % (thread_id, sys.getrefcount(frame))
                        traceback.print_stack(frame)    
                sys.exit(-1)
        # look for garbage
        gc.collect()
        print repr(gc.garbage)


    def tearDown(self):
        for i in range(1, ccount):
            self.delete(i)
        print "deleted %d files" % ccount
    
    
if __name__ == "__main__":
    testbase.runTests(unittest.findTestCases(__import__('__main__')))        
    



# $Id: ApacheHandler.py 2028 2006-01-16 23:39:04Z zzzeek $
# ApacheHandler.py - handles apache requests for Myghty
# Copyright (C) 2004, 2005 Michael Bayer mike_mp@zzzcomputing.com
#
# This module is part of Myghty and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php
#
#

from mod_python import apache
from mod_python import util as mputil
import myghty.exception as exception
import myghty.buffer
import re,string, types, time
from myghty.util import *
from myghty.resolver import *
import myghty.http.HTTPHandler as HTTPHandler

def handle(httpreq, interpreter_name = None, **params):
    return get_handler(httpreq, interpreter_name, **params).handle(httpreq = httpreq, **params)

def get_handler(httpreq, interpreter_name = None, **params):
    if interpreter_name is None:
        try:
            interpreter_name = httpreq.get_options()['MyghtyInterpreterName']
        except KeyError:
            pass
    return HTTPHandler.get_handler(ApacheHandler, httpreq = httpreq, interpreter_name = interpreter_name, **params)

class ApacheHandler(HTTPHandler.HTTPHandler):
    def __init__(self, httpreq, **params):
        HTTPHandler.HTTPHandler.__init__(self, httpreq, LogBuffer(httpreq.server), **params)

    def do_handle_result(self, httpreq, status_code, reason):
        httpreq.status = HTTPHandler.HTTP_OK
        if status_code == HTTPHandler.HTTP_OK:
            return apache.OK
        else:
            return status_code

    def do_get_init_params(self, httpreq, **params):
        table = httpreq.get_options()
        
        options = params.copy()
        for key in table.keys():
            match = re.match("^Myghty(.*)$", key)
            if not match: continue
            param = match.group(1)
            param = string.lower(re.sub(r"(\w)([A-Z])", r"\1_\2", param))
            if not options.has_key(param):
                options[param] = eval(table[key])
            
        return options
    
    def do_make_request_impl(self, httpreq, **params):
        httpreq.add_common_vars()
        return ApacheRequestImpl(httpreq, **params)

    def do_get_resolver(self, **params):
        return ApacheResolver(**params)
    
    def do_get_component(self, httpreq, **params):
        return httpreq.uri
        
            


class LogBuffer(object):
    def __init__(self, server, retcode = apache.APLOG_NOTICE):
        self.server = server
        self.retcode = retcode

    def write(self, s):
        apache.log_error(string.rstrip(s), self.retcode, self.server)

    def writelines(self, list):
        for line in list:
            apache.log_error(string.rstrip(line), self.retcode, self.server)

    def flush(self):
        pass

class ApacheBuffer(HTTPHandler.HTTPWriter):

    def send_headers(self):
        if self.headers_sent:
            return

        self.headers_sent = True

        if not self.httpreq._content_type_set:
            self.httpreq.content_type = 'text/html'

        # this method is only documented in mod py 2.7
        # (where its needed) but seems harmless though useless
        # to use in mod py 3.0 as well
        self.httpreq.send_http_header()


    def writelines(self, list):
        for line in list:
            self.write(line + "\n")

        def flush(self):pass

class ApacheResolver(HTTPHandler.HTTPResolver):pass

class ApacheRequestImpl(HTTPHandler.HTTPRequestImpl):
    def __init__(self, httpreq, use_modpython_session = False, **params):

        self.use_modpython_session = use_modpython_session

        HTTPHandler.HTTPRequestImpl.__init__(self, httpreq, LogBuffer(httpreq.server),
            LogBuffer(httpreq.server, apache.APLOG_ERR), **params)
        
        
    def get_session(self, **params):
        if not hasattr(self, 'session'):
            if self.use_modpython_session:
                import mod_python.Session
                self.session = mod_python.Session.Session(timeout = self.session_args.timeout)
            else:
                self.session = self.session_args.get_session(self.httpreq, **params)
            
        return self.session

    def do_get_out_buffer(self, httpreq, out_buffer = None, **params):
        if out_buffer is None:
            out_buffer = httpreq
            
        return ApacheBuffer(httpreq, out_buffer = out_buffer, **params)

    def do_make_request_args(self, httpreq, **params):
        """given a mod_python request, returns a real dict of key values, where the values are
        one of:  a string, a list of strings, a Field object with an upload value, a list of
        Field objects with upload values."""

        def formatfield(field):
            if type(field) == types.ListType:
                return map(formatfield, field)
            elif isinstance(field, mputil.Field):
                return field
            else:
                return str(field)

        request_args = {}
        fields = mputil.FieldStorage(httpreq, keep_blank_values = True)
        for key in fields.keys():
            request_args[key] = formatfield(fields[key])
        return request_args
        






import sys, StringIO, unittest
from myghty import request, interp
import testbase

class LoggingRequestImpl(request.DefaultRequestImpl):
    def __init__(self, logger=sys.stderr, **params):
        request.DefaultRequestImpl.__init__(self, **params)
        self.logger = logger

class TestRequestLogging(unittest.TestCase):
    def setUp(self):
        self.logger = StringIO.StringIO()
        self.m = interp.Interpreter().make_request(
            None,
            request_impl=LoggingRequestImpl(logger=self.logger))

    logOutput = property(lambda self: self.logger.getvalue())
    
    def testLog(self):
        self.m.log('howdy')
        self.failUnlessEqual(self.logOutput, "howdy\n")

    def testLogger(self):
        self.m.logger.write('foo')
        self.failUnlessEqual(self.logOutput, "foo\n")

    def testLoggerWritelines(self):
        self.m.logger.writelines(['foo', 'bar'])
        self.failUnlessEqual(self.logOutput, "foo\nbar\n")

class TestWarningsGetLogged(testbase.ComponentTester):
    def test(self):
        log = StringIO.StringIO()
        self.runComponent('''% import warnings
                             % warnings.warn("Boo!")
                             ''',
                          config={'request_impl':
                                  LoggingRequestImpl(logger=log)})
        output = log.getvalue()
        self.failUnlessEqual(output[output.rindex('.myt'):],
                             '.myt:2: UserWarning: Boo!\n'
                             '  % warnings.warn("Boo!")\n')

class TestWarningsGetLoggedMemory(TestWarningsGetLogged):
    """Same as above, except with no compile cache.
    """
    # In this case, the keys in interpreter.reverse_lookup
    # start with 'memory:'.

    config = dict(data_dir=None)

    def testSanity(self):
        self.runComponent(' ')
        self.failUnless(filter(lambda s: s.startswith('memory:'),
                               self.interpreter.reverse_lookup.keys()))
        

if __name__ == '__main__':
    unittest.main()



import testbase
import myghty.resolver as resolver
import myghty.exception as exception
import unittest, sys, string

class FileResolveTest(testbase.MyghtyTest):
    def class_set_up(self):
        
        self.resolver = resolver.Resolver(
            component_root = [{'htdocs': self.htdocs}, {'comp': self.components}],
            use_static_source = True
        )
        
        self.create_file(self.htdocs, 'hello.myt', "hello component")
        self.create_file(self.components, 'searchup.myt', "searchup component")
        self.create_directory(self.htdocs, './foo/lala')
        self.create_file(self.htdocs, './foo/searchup2.myt', "<html> foo</html>")
        self.create_file(self.htdocs, './foo/lala/hi.myt', "im hi.myt")
        self.create_file(self.components, 'dhandler', "im a dhandler")

    def class_tear_down(self):
        self.remove_file(self.components, "dhandler")
        self.remove_file(self.components, "searchup.myt")
        self.remove_file(self.htdocs, './foo/searchup2.myt')
        self.remove_file(self.htdocs, './foo/lala/hi.myt')
        self.remove_file(self.htdocs, 'hello.myt')
        
    def testcomponent(self):
        "locates a component in the root directory"
        resolution = self.resolver.resolve('/hello.myt', search_upwards = False, raise_error = True)
        csource = resolution.csource
        dumpresolution(resolution)
        self.assert_(csource is not None and csource.path == '/hello.myt' and csource.dir_name == '/', "hello.myt not located")

    def testdircomponent(self):
        "locates a component in a subdirectory"
        resolution = self.resolver.resolve('/foo/lala/hi.myt', search_upwards = False, raise_error = True)
        csource = resolution.csource
        dumpresolution(resolution)
        self.assert_(csource is not None and csource.path == '/foo/lala/hi.myt' and csource.dir_name == '/foo/lala' and csource.name=='hi.myt', "hi.myt not properly located")
    
    def testcomponentnoslash(self):
        "locates a component with no directory information in the URI"
        resolution = self.resolver.resolve('hello.myt', search_upwards = False, raise_error = True)
        csource = resolution.csource
        dumpresolution(resolution)
        self.assert_(csource is not None, "hello.myt not located")
        self.assert_(csource.path == '/hello.myt' and csource.dir_name == '/', "hello.myt has wrong path info, path %s dir_name %s" % (csource.path, csource.dir_name))

    def testcache(self):
        "locates a component twice with caching on, checks that the second resolution succeeded"
        resolution = self.resolver.resolve('/hello.myt', search_upwards = False, raise_error = True)
        csource = resolution.csource
        dumpresolution(resolution)
        resolution = self.resolver.resolve('/hello.myt', search_upwards = False, raise_error = True)
        csource = resolution.csource
        dumpresolution(resolution)
        self.assert_(csource is not None and csource.path == '/hello.myt' and csource.dir_name == '/', "hello.myt not located")

    
    def testupwardsnonexistentdir(self):
        "locates a component searching upwards, with the initial directory nonexistent"
        resolution = self.resolver.resolve('/foo/bar/searchup.myt', search_upwards = True, raise_error = True)
        csource = resolution.csource
        dumpresolution(resolution)
        self.assert_(csource is not None and csource.path == '/searchup.myt' and csource.dir_name == '/', "search upwards test #1 failed")
        
    def testupwardsrealdir(self):
        "locates a component searching upwards, with a real initial directory"
        resolution = self.resolver.resolve('/foo/lala/searchup2.myt', search_upwards = True, raise_error = True)
        csource = resolution.csource
        dumpresolution(resolution)
        self.assert_(csource is not None and csource.path == '/foo/searchup2.myt' and csource.dir_name == '/foo', "search upwards test #2 failed, " + csource.path + " " + csource.dir_name)

    def testnotfound(self):
        "tries to locate a component that doesnt exist"
        resolution = self.resolver.resolve('/foo/hi.myt', search_upwards = False, raise_error = False)
        self.assert_(resolution is None, "not found test failed")
        
    def testupwardsnotfound(self):
        "tries to locate a component searching upwards that does not exist"
        resolution = self.resolver.resolve('/foo/autohandler', search_upwards = True, raise_error = False)
        self.assert_(resolution is None, "not found upwards test failed")

    def testupwardsnoslash(self):
        "locates a component upwards, where the uri has no directory information"
        resolution = self.resolver.resolve('autohandler', search_upwards = True, raise_error = False)
        self.assert_(resolution is None, "not found upwards test with no slashes failed")


    def testdhandler(self):
        "locates a dhandler"
        resolution = self.resolver.resolve('/foo/bar/lala/foo.myt', enable_dhandler = True, raise_error = False)
        dumpresolution(resolution)
        self.assert_(resolution is not None, "dhandler test failed")
        csource = resolution.csource
        self.assert_(csource.path == '/dhandler' and csource.dir_name == '/', "dhandler test failed")
        self.assert_(resolution.dhandler_path == 'foo/bar/lala/foo.myt', "dhandler_path is %s" % resolution.dhandler_path)

    def testdhandlernoslash(self):
        "attempts to locate a dhandler with an initial uri that has no directory information, and no dhandler"
        self.remove_file(self.components, "dhandler")
        resolution = self.resolver.resolve('hoho.myt', enable_dhandler = True, raise_error = False)
        self.assert_(resolution is None, "dhandler test with no slashes failed")
        
    def testdir(self):
        "attempts to locate a component with a uri that resolves to a real directory"
        try:
            resolution = self.resolver.resolve('/foo/lala/', search_upwards = False, raise_error = True)
            self.assert_(False, "didnt detect directory")
        # we sort of liked when this was PathIsDirectory
        except exception.ComponentNotFound, e:
            #print e.message()
            self.assert_(True)
        

class PathModuleResolveTest(testbase.MyghtyTest):
    def class_set_up(self):
        self.create_file(self.components, 'pathmodule1.py', 
"""
import myghty.component as component

def dostuff(m):
    pass

    
""")

        self.create_file(self.lib, 'pathmodule2.py', 
"""
import myghty.component as component

def lala(m):
    pass

""")

        pm2 = __import__('pathmodule2')
        
        self.create_directory(self.components, 'foo/bar')

        self.create_file(self.components, 'foo/bar/pathmodule3.py', 
"""
import myghty.component as component

def dostuff(m):
    pass

def index():
    pass

class lala:
    def __call__():
        pass
        
    def foofoo(m):
        pass
        
bar = lala()
""")

        self.create_file(self.components, 'foo/dhandler.py', 
"""
import myghty.component as component

def dostuff(m):
    pass

def index(m):
    pass
    
class lala:
    def __call__():
        pass
        
hoho = lala()
""")

        self.resolver = resolver.Resolver(
            module_root = [
                # a file path
                self.components,
            
                # an actual module
                pm2, 
                
            ],
        )            

    def class_tear_down(self):
        self.remove_file(self.components, 'pathmodule1.py')
        self.remove_file(self.components, 'pathmodule2.py')
        self.remove_file(self.components, 'foo/dhandler.py')
        self.remove_file(self.components, 'foo/bar/pathmodule3.py')

    def testpathmod1(self):
        resolution = self.resolver.resolve('/pathmodule1/dostuff', raise_error = True)
            
        dumpresolution(resolution)
        csource = resolution.csource
        self.assert_(csource is not None)

    def testpathmod2(self):
        resolution = self.resolver.resolve('/lala', raise_error = False)
        dumpresolution(resolution)
        csource = resolution.csource
        self.assert_(csource is not None)

    def testpathmod3(self):
        resolution = self.resolver.resolve('/foo/bar/pathmodule3/dostuff', raise_error = False)
        dumpresolution(resolution)
        csource = resolution.csource
        self.assert_(csource is not None)

    def testpathmod4(self):
        resolution = self.resolver.resolve('/foo/bar/pathmodule3/lala', raise_error = False)
        dumpresolution(resolution)
        csource = resolution.csource
        self.assert_(csource is not None)

    def testpathmod5(self):
        resolution = self.resolver.resolve('/foo/hoho', raise_error = False, enable_dhandler = True)
        dumpresolution(resolution)
        csource = resolution.csource
        self.assert_(csource is not None)

    def testpathmod6(self):
        resolution = self.resolver.resolve('/foo/bar/pathmodule3/', raise_error = False)
        dumpresolution(resolution)
        csource = resolution.csource
        self.assert_(csource is not None)

    def testpathmod7(self):
        resolution = self.resolver.resolve('/foo/bar/pathmodule3/bar/foofoo', raise_error = False)
        dumpresolution(resolution)
        csource = resolution.csource
        self.assert_(csource is not None)




class ModuleResolveTest(testbase.MyghtyTest):
    def class_set_up(self):
        self.create_file(self.lib, '__init__.py', "")
        self.create_file(self.lib, 'mymodule1.py', 
"""
import myghty.component as component
  
def mod1dostuff(m):
    pass
    
class TestComp(component.ModuleComponent):
  
   def do_component_init(self, **params):
      pass               
  
   def do_run_component(self, m, ARGS, **params):
      pass
""")

        self.create_file(self.lib, 'mymodule2.py', 
"""
import myghty.component as component

def dostuff(m):
    pass

def index(m):
    pass
    
class TestComp(component.ModuleComponent):
  
   def do_component_init(self, **params):
      pass               
  
   def do_run_component(self, m, ARGS, **params):
      pass
      
class foo(object):
    def run(self, m):
        pass

lala = foo()
""")


        self.mod2 = __import__('mymodule2')

        self.resolver = resolver.Resolver(
            module_components = [
                {'/index.myt' : 'mymodule1:TestComp'}, 
                {'/foo/.*': self.mod2.TestComp},
                {'/bar/.*': "mymodule2:dostuff"},
                {'/foo2/.*': self.mod2.lala.run},
                {'/foo3/.*': "mymodule2:lala.run"},
            ],
        )
    
    def class_tear_down(self):
        self.remove_file(self.lib, '__init__.py')
        self.remove_file(self.lib, 'mymodule1.py')
        self.remove_file(self.lib, 'mymodule1.pyc')
        self.remove_file(self.lib, 'mymodule2.py')
        self.remove_file(self.lib, 'mymodule2.pyc')
        try: del sys.modules['mymodule1']
        except KeyError:pass
        try: del sys.modules['mymodule2']
        except KeyError:pass    


    def testresolvestring(self):
        resolution = self.resolver.resolve('/index.myt', search_upwards = False, raise_error = False)
        csource = resolution.csource
        dumpresolution(resolution)
        self.assert_(csource is not None and csource.id == 'module|mymodule1:class:TestComp', "unexpected id: " + csource.id)
        self.assert_(sys.modules.has_key('mymodule1'), 'module1 module not loaded into sys.modules')

    def testresolvemod(self):
        resolution = self.resolver.resolve('/foo/lala.myt', search_upwards = False, raise_error = False)
        csource = resolution.csource
        dumpresolution(resolution)
        self.assert_(csource is not None and csource.id == 'module|mymodule2:class:TestComp', "unexpected id: " + csource.id)
        self.assert_(sys.modules.has_key('mymodule2'), 'module2 module not loaded into sys.modules')
    
    def testresolvemodfunction(self):
        resolution = self.resolver.resolve('/bar/lala', raise_error = False)
        csource = resolution.csource
        dumpresolution(resolution)
        self.assert_(csource is not None)
        self.assert_(sys.modules.has_key('mymodule2'), 'module2 module not loaded into sys.modules')
        
    def testresolvemodmethod(self):
        resolution = self.resolver.resolve('/foo2/lala', raise_error = False)
        csource = resolution.csource
        dumpresolution(resolution)
        self.assert_(csource is not None)

    def testresolvemodmethodstring(self):
        resolution = self.resolver.resolve('/foo3/lala', raise_error = False)
        csource = resolution.csource
        dumpresolution(resolution)
        self.assert_(csource is not None)

        
    def testnotfound(self):
        resolution = self.resolver.resolve('/foob/lala.myt', search_upwards = True, raise_error= False)
        self.assert_(resolution is None)
            

def dumpresolution (resolution):
    # TODO: echo flags
    if False:
        print "\nID: " + resolution.csource.id
        print "resolve: " + string.join(resolution.detail, ',')
        print "match: " + repr(resolution.match)
    
if __name__ == '__main__':
    runner = unittest.TextTestRunner(verbosity=2, descriptions=False)
    unittest.main(testRunner=runner)        



"""
    a set of classes that represent an HTML form, including its field names, 
    the values of those fields corresponding to their application setting
    as well as the value pulled from an HTTP request, and validation rules
    for the fields.  "hierarchical forms" can be created as well, allowing a form
    to be grouped into "subforms", which may live on the same HTML page or across
    several HTML pages.
"""

from myghty.util import *
import inspect, datetime, types

class FormElement(object):
    def __init__(self, name,  **params):
        self.name = name
        self.description = ''
        self.errors = []
        self.isvalid = None
    def set_form(self, form):pass
    def set_request(self, req, validate = True):pass

    def append_error(self, error):
        self.errors.append(error)
        self.isvalid=False
    
    def clear(self):pass
    def is_valid(self):return self.isvalid
    def unvalidate(self):pass
    
class Form(FormElement):
    def __init__(self, name, elements, **params):
        FormElement.__init__(self, name)
        self.success = []
        
        self.elements = OrderedDict()
        for elem in elements:
            self.elements[elem.name] = elem
            elem.set_form(self)

    def append_success(self, success):
        self.success.append(success)

    def set_form(self, form):pass
        
    def append(self, element):
        self.elements[element.name] = element
        element.set_form(self)
        
    def __iter__(self):
        return iter(self.elements.values())

    def get(self, key, *args, **kwargs):
        return self.elements.get(key, *args, **kwargs)
    def __getitem__(self, key):
        return self.elements[key]
    def __setitem__(self, key, elem):
        self.elements[elem.name] = elem
        elem.set_form(self)
        
    def unvalidate(self):
        self.isvalid = None
        for elem in self.elements.values(): elem.unvalidate()
    
    def set_request(self, req, validate = True):
        self.isvalid = True
        for elem in self.elements.values():
            if elem is None:
                continue
            elem.set_request(req, validate)
            if not elem.is_valid():
                self.isvalid = False

    def _fieldname(self, formfield):
        return formfield.name

    def reflect_from(self, object):
        for elem in self.elements.values():
            elem.reflect_from(object)
        
    def reflect_to(self, object):
        for elem in self.elements.values():
            elem.reflect_to(object)

    def clear(self):
        for elem in self.elements.values():
            elem.clear()
    
class SubForm(Form):
    def _fieldname(self, formfield):
        return self.name + "_" + formfield.name
        
class FormField(FormElement):
    def __init__(self, name, description=None, required=False, default=None, data=None, attribute=False, setattribute=False, getattribute=False, disabled=False):
        FormElement.__init__(self, name)

        if description is None:
            self.description = name
        else:
            self.description = description
        
        self.required = required

        if setattribute is not False:
            self.setattribute = setattribute
        elif attribute is not False:
            self.setattribute = attribute
        else:
            self.setattribute = name
        if getattribute is not False:
            self.getattribute = getattribute
        elif attribute is not False:
            self.getattribute = attribute
        else:
            self.getattribute = name

        self.default = default
        self.value = default
        self.displayname = None
        self.data = data
        self.disabled = disabled
    
    def clear(self):
        self.value = self.default
        
    def set_form(self, form):
        self.displayname = form._fieldname(self)

    def _set_value(self, value):
        self._value = value
        if value is None:
            self.display = ''
        else:
            self.display = str(value)

    value = property(lambda s: s._value, lambda s, v: s._set_value(v))
    
    def reflect_to(self, object):
        if self.disabled or self.setattribute is None:
            return
        elif callable(self.setattribute):
            self.setattribute(self, object)
        else:
            attrs = self.setattribute.split('.')
            o = object
            for attr in attrs[0:-2]:
                o = getattr(o, attr)
            setattr(o, attrs[-1], self.value)

    def reflect_from(self, object):
        if self.getattribute is None:
            return
        elif callable(self.getattribute):
            self.value = self.getattribute(object)
        else:
            for attr in self.getattribute.split('.'):
                try:
                    object = getattr(object, attr)
                except AttributeError:
                    return
            self.value = object

    def set_request(self, request, validate = True):
        """sets the request for this form.   if validate is True, also 
        validates the value."""

        if request.has_key(self.displayname):
            self.display = request[self.displayname]
            
        if validate:
            if self.required and (not request.has_key(self.displayname) or not self.display):
                self.isvalid = False
                self.append_error('required field "%s" missing' % self.description)
            else:
                self.isvalid = True
                self.value = self.display
                
    def unvalidate(self):
        """resets the isvalid state of this form to None"""
        self.isvalid = None
        self.errors = []
        

class IntFormField(FormField):

    def _set_value(self, value):
        if not value:
            self._value = None
            self.display = ''
        else:
            try:
                self._value = int(value)
            except ValueError:
                self.append_error('field "%s" must be an integer number' % self.description)
                self.isvalid = False
            self.display = str(value)
            
                
class CompoundFormField(SubForm):
    """
    a SubForm that acts like a single formfield in that it contains a single value,
    but also contains subfields that comprise elements of that value.
    
    examples: a date with year, month, day fields, corresponding to a date object
    
    more exotic examples: a credit card field with ccnumber, ccexpiration fields corresponding to a 
    CreditCard object, an address field with multiple subfields corresopnding to an Address object, etc.
    
    """

    def _set_value(self, value):
        SubForm._set_value(self, value)
        self.set_compound_values(value)        

    def set_compound_values(self, value):
        pass
        


class CCFormField(FormField):

    def _set_value(self, value):
        if not self.luhn_mod_ten(value):
            self.isvalid = False
            self.append_error('invalid credit card number')
        else:
            self._value = value
   
    def luhn_mod_ten(self, ccnumber):
        """ checks to make sure that the card passes a luhn mod-10 checksum.
        
        courtesy: http://aspn.activestate.com
        """
        sum = 0
        num_digits = len(ccnumber)
        oddeven = num_digits & 1

        for count in range(0, num_digits):
            digit = int(ccnumber[count])

            if not (( count & 1 ) ^ oddeven ):
                digit = digit * 2
            if digit > 9:
                digit = digit - 9

            sum = sum + digit

        return ( (sum % 10) == 0 )
    
    
class DateFormField(CompoundFormField):
    # TODO: fixy
    def __init__(self, name, fields, yeardeltas = range(-5, 5), required = False, *args, **params):

        elements = {}
        
        for field in fields:
            if field == 'ampm':
                elements[field] = FormField(field, required = required)
            else:
                elements[field] = IntFormField(field, required = required)

        for key in ['year', 'month', 'day']:
            if elements.has_key(key):
                self.hasdate = True
                break
        else:
            self.hasdate = False

        for key in ['hour', 'minute', 'second']:
            if elements.has_key(key):
                self.hastime = True
                break
        else:
            self.hastime = False

            
        assert (self.hasdate or self.hastime)
        
        CompoundFormField.__init__(self, name, elements.values(), **params)

        self.required = required
        
        if self.hasdate:
            today = datetime.datetime.today()
            year = today.year
            self.yearrange = [year + i for i in yeardeltas]


    def set_compound_values(self, value):
        if self.hasdate:
            self.elements['year'].value = value.year
            self.elements['month'].value = value.month
            self.elements['day'].value = value.day
            
        if self.hastime:
            if self.elements.has_key('ampm'):
                v = value.hour % 12
                if v == 0: v = 12
                self.elements['hour'].value = v
            else:
                self.elements['hour'].value = value.hour
            self.elements['minute'].value = value.minute
            self.elements['second'].value = value.second
            if self.elements.has_key('ampm'):
                self.elements['ampm'].value = (value.hour > 12 and 'pm' or 'am')
        
    def set_request(self, request, validate = True):
        CompoundFormField.set_request(self, request, validate)
        if validate:
            for elem in self.elements.values():
                if elem.is_valid() is False:
                    self.append_error('field "%s": %s' % (self.description, string.join([elem.errors])))
                    return

            args = {}

            has_value = False
            if self.hasdate:
                dummy = datetime.date.min
                for key in ['year', 'month', 'day']:
                    if self.elements.has_key(key):
                        args[key] = self.elements[key].currentvalue
                        if args[key] is not None:
                            has_value = True
                    else:
                        args[key] = getattr(dummy, key)
                
            if self.hastime:
                dummy = datetime.time.min
                for key in ['hour', 'minute', 'second']:
                    if self.elements.has_key(key):
                        args[key] = self.elements[key].currentvalue
                        if args[key] is not None:
                            has_value = True
                    else:
                        args[key] = getattr(dummy, key)
                
                if self.elements.has_key('ampm'):
                    if self.elements['ampm'] == 'pm':
                        args['hour'] += 12
                    elif args['hour'] == 12:
                        args['hour'] = 0

            if not has_value:
                self.request_value = None
                return
                
            try:
                if self.hasdate and self.hastime:
                    value = datetime.datetime(**args)
                elif self.hasdate:
                    value = datetime.date(**args)
                else:
                    value = datetime.time(**args)
                self.request_value = value
                self.currentvalue = value
                self.isvalid = True
            except TypeError, e:
                self.isvalid = False
                self.currentvalue = None
                self.append_error('field "%s" does not contain a valid date/time (%s)' % (self.description, str(e)))
            except ValueError, e:
                self.isvalid = False
                self.currentvalue = None
                self.append_error('field "%s" does not contain a valid date/time (%s)' % (self.description, str(e)))
            
            



from ez_setup import use_setuptools
use_setuptools()
from setuptools import setup, find_packages

setup(name = "Myghty",
    version = "1.1",
    description = "View/Controller Framework and Templating Engine",
    author = "Mike Bayer",
    author_email = "mike@myghty.org",
    url = "http://www.myghty.org",
    download_url = "http://www.myghty.org/links.myt?linkid=download",
    package_dir = {'':'lib'},
    packages = find_packages('lib'),
    license = "MIT License",
    long_description = """\
A Python-based template and view-controller framework derived from HTML::Mason. Supports the full featureset of Mason, allowing component-based web development with Python-embedded HTML, and includes many new concepts and features not found in Mason.

`Development SVN <http://svn.myghty.org/myghty/trunk#egg=myghty-dev>`_

""",
    classifiers = ["Development Status :: 5 - Production/Stable",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: MIT License",
    "Programming Language :: Python",
    "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    "Topic :: Text Processing :: Markup"
    ],
    package_data={
            'myghty': [
                'paster_templates/*/*.py', 
                'paster_templates/*/*_tmpl', 
                'paster_templates/*/+package+/*.py', 
                'paster_templates/*/+package+/*.my?', 
                'paster_templates/*/+package+/*_tmpl', 
                'paster_templates/*/+package+/*/*.py', 
                'paster_templates/*/+package+/*/*.my?', 
                'paster_templates/*/+package+/*/*_tmpl', 
                'paster_templates/*/+package+/*/*.css', 
                'paster_templates/*/+package+/*/*.png', 
                'paster_templates/*/+package+/*/*handler', 
                ],
    },
    install_requires=["Routes >= 1.0", "Paste", "PasteDeploy", "PasteScript"],

    entry_points="""
    [paste.paster_create_template]
    myghty_routes=myghty.paste.templates:RoutesTemplate
    myghty_simple=myghty.paste.templates:SimpleTemplate
    myghty_modulecomponents=myghty.paste.templates:MCTemplate
    """,
    )





from zblog.controller import *
import re, string
from zblog.domain.blog import *
import zblog.domain.actions as actions
import zblog.util.form as formutil
import zblog.database.mappers as mapper
from zblog.controller.blog.comments import index as commentcontroller

class BlogHome(Controller):
    @access_control()
    def __call__(self, m, ARGS):
        """shows all the posts within a blog"""
        match = re.search(r'/(\w+)/(?:topic/(\w+)/)?$', m.request_path)
        if match:
            blog_id = match.group(1)
            if blog_id == 'my':
                self.template(m, '/index.myt', blog_user=self.get_user(m))
                return
            keyword = match.group(2) or False
        else:
            m.abort(404)

        blog = Blog.mapper.get(blog_id)
        if blog is None:
            m.abort(404)

        self.template(m, '/blog/index.myt', blog=blog, keyword=keyword)

    @access_control(action=actions.CreatePost())
    def post(self, m, ARGS, blog_id, post_id=None, preview=False):
        """submits a post, or previews it."""
        blog = Blog.mapper.get(blog_id)
        form = self.postform(m, ARGS)
        form.set_request(ARGS, validate=True)
        
        if post_id:
            post = Post.mapper.get(post_id)
            if post is None:
                form.append_error("no post found for id '%s'" % post_id)
                self.template(m, '/blog/index.myt', blog=blog, loadcomponent='/blog/forms.myt:postform', form=form)
                return
        else:
            post = None
            
        if not form.is_valid():
            self.template(m, '/blog/index.myt', blog=blog, loadcomponent='/blog/forms.myt:postform', form=form, post=post)
        elif int(preview):
            if post is None:
                preview = Post()
            else:
                preview = post
            self.reflect_to(m, form, preview, blog)
            self.template(m, '/blog/index.myt', blog=blog, loadcomponent='/blog/forms.myt:postform', form=form, preview=preview, post=post)
        else:
            mapper.begin()
            if post is None:
                post = Post()
            self.reflect_to(m, form, post, blog)
            mapper.commit()
            if not post_id:
                form.append_success("Post added")
            else:
                form.append_success("Post updated")
            self.template(m, '/blog/index.myt', blog=blog, post=post, loadcomponent='/blog/post.myt', viewcomments=False, form=form )

    def reflect_to(self, m, form, post, blog):
        form.reflect_to(post)
        primary = True
        
        post.topics = []
        for keyword in form['topic_keywords'].value.split(' '):
            if not keyword:
                continue
            topic = Topic.mapper.get_by(keyword=keyword)
            if topic is None:
                topic = Topic(keyword=keyword, description=keyword)
            post.topics.append(TopicAssociation(post=post, topic=topic, is_primary=False))
        
        print repr([(t, t.post, t.topic) for t in post.topics.records.keys()])
            
        post.user = self.get_user(m)
        post.blog = blog

    def reflect_from(self, form, post):
        form.reflect_from(post)
        form['topic_keywords'].value = string.join([t.topic.keyword for t in post.topics])

    @access_control(action=actions.CreatePost())
    def new_post(self, m, ARGS, blog_id):
        """provides a blank "submit a post" screen"""
        blog = Blog.mapper.get(blog_id)
        form = self.postform(m, ARGS)
        self.template(m, '/blog/index.myt', loadcomponent='/blog/forms.myt:postform', form=form, blog=blog)
            
    @access_control(action=actions.EditPost())
    def ajax_edit_post(self, m, ARGS, post_id):
        """produces an "edit post" screen, given the id of the post"""
        post = Post.mapper.get(post_id)
        form = self.postform(m, ARGS)
        self.reflect_from(form, post)
        m.comp('/blog/forms.myt:postform', form=form, blog=post.blog, post=post)

    @access_control(action=actions.EditPost())
    def ajax_delete_post(self, m, ARGS, post_id, confirm=False):
        """deletes a post, given its ID."""
        post = Post.mapper.get(post_id)
        blog = post.blog
        if not confirm:
            m.comp('/blog/forms.myt:delete_confirm', post=post)
            return
        form = self.postform(m, ARGS)
        mapper.begin()
        mapper.delete(post)
        mapper.commit()
        form.append_success("Post deleted")
        m.comp('/blog/index.myt:postlist', blog=blog, form=form)

    @access_control()
    def view(self, m, ARGS):
        """views a specific post within a blog as well as its comments"""
        match = re.search(r'/(\w+)/$', m.request_path)
        if match:
            post_id = match.group(1)
        else:
            m.abort(404)
        
        #post = Post.mapper.options(nodefer('body')).get(post_id)
        post = Post.mapper.get(post_id)
        if post is None:
            m.abort(404)

        commentform = commentcontroller.commentform(m, ARGS, post=post)
        self.template(m, '/blog/index.myt', blog=post.blog, post=post, showcomments=True, form=commentform, loadcomponent='/blog/post.myt' )
        
    def postform(self, m, ARGS):
        form=formutil.Form('post', [
            formutil.IntFormField('blog_id'),
            formutil.IntFormField('post_id'),
            formutil.FormField('preview'),
            formutil.FormField('headline', required=True),
            formutil.FormField('topic_keywords'),
            formutil.FormField('summary'),
            formutil.FormField('body', required=True)
        ])
        if ARGS.has_key('post_id'):
            form['post_id'].value = ARGS['post_id']
        if ARGS.has_key('blog_id'):
            form['blog_id'].value = ARGS['blog_id']
        return form
        
index = BlogHome()




# $Id: request.py 2133 2006-09-06 18:52:56Z dairiki $
# request.py - handles component calls for Myghty templates
# Copyright (C) 2004, 2005 Michael Bayer mike_mp@zzzcomputing.com
# Original Perl code and documentation copyright (c) 1998-2003 by Jonathan Swartz.
#
# This module is part of Myghty and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php
#
"""The request package and its primary class, Request, provide the "m" object in Myghty templates.
The Request is instantiated by the Interpreter and handles component execution state as well
as buffered state.
It provides the interface to all actions taken by components, such as writing output,
calling other components, calling subrequests, and controllling request state.  Also
provides the programmatic interface to key objects such as caches and sessions.
"""

from myghty.util import *
import myghty.buffer
from myghty import exception
import sys, string, re, StringIO, types, os, time, warnings
import posixpath as unixpath
from myghty.container import CreationAbortedError
import myghty.escapes as escapes
import myghty.csource as csource
from myghty.requestbuffer import UnicodeRequestBuffer, StrRequestBuffer

DEFAULT_OUTPUT_ENCODING = sys.getdefaultencoding()
DEFAULT_ENCODING_ERRORS = 'strict'

# current instance of Request for this thread. 
# this is flaky as the request module seems to get reimported when it is
# first imported within a generated component
reqinstance = ThreadLocal()

__all__ = ['Request', 'instance', 'threadlocal']
class Request(object):

    """request object, calls components and directs output.  also is the 
    primary programmatic interface presented to a template's environment."""

    def __init__(self,
        interpreter,
        component = None,
        request_impl = None,
        max_recursion = 32,
        auto_flush = False,
        use_dhandlers = True,
        dont_auto_flush_filters = False,
        resolver_context = 'request',
        parent_request = None,
        request_depth = 0,
        request_path = None,
        raise_error = False,
        disable_wrapping = False,
        output_encoding = DEFAULT_OUTPUT_ENCODING,
        encoding_errors = DEFAULT_ENCODING_ERRORS,
        disable_unicode = False,
        **params
        ):
        """init is called internally by the Interpreter."""
        
        self.component = component
        self.auto_flush = auto_flush
        self.executed = False
        self.interpreter = interpreter
        self.execution_stack = []
        self.max_recursion = max_recursion
        self.dont_auto_flush_filters = dont_auto_flush_filters
        self.use_dhandlers = use_dhandlers
        self.disable_wrapping = disable_wrapping
        self.raise_error = raise_error
        self.parent_request = parent_request

        self._attributes = {}

        self.request_depth = request_depth
        
        self.current_csource = None

        self.starttime = time.time()

        self.comphash = {}
        self.compcaches = {}

        self.request_path = request_path

        self.resolver_context = resolver_context
        
        if parent_request is not None:
            self.root_request = parent_request.root_request
            self.declined_components = parent_request.declined_components
            self.dhandler_path = parent_request.dhandler_path
        else:
            self.declined_components = {}
            self.root_request = self
            self.dhandler_path = None

        if request_impl:
            self.request_impl = request_impl
        else:
            self.request_impl = DefaultRequestImpl(**params)

        self.buffer = None
        self.__output_encoding = output_encoding, encoding_errors
        self.disable_unicode = bool(disable_unicode)
        
    notes = property(lambda self: self.attributes, doc="""A synonym for m.attributes""")

    root_request_path = property(lambda self:self.root_request.request_path, doc="""\
    The URI sent to the ultimate root request of this Request.
    """)
    root_request_args = property(lambda self:self.root_request.request_args, doc="""\
    The request argument dictionary sent to the ultimate root request of this Request.
    """)

    def _parentattr(self):
        if self.parent_request is not None:
            return self.parent_request.attributes
        else:
            return None

    attributes = property(lambda self:InheritedDict(self._attributes, lambda: self._parentattr()), doc="""\
    A dictionary where arbitrary attributes can be stored and retrieved.  Inherits data from its 
    parent request, if any.
    """)
            
    def is_subrequest(self):
        """returns True if this request is a subrequest."""
        return self.parent_request is not None
        
    def clone(self, **params):
        """creates a copy of this Request.  Normally used to create subrequests."""
        if not params.has_key('request_impl'):
            params['request_impl'] = self.request_impl.clone(**params)
        clone = ConstructorClone(self, **params)
        return clone.clone()
        
    
    class StackFrame:
        """data object representing the currently executing component context.
        
        gets pushed and popped to/from the execution_stack instance variable.
        """
        
        def __init__(self, request, component, args, base_component, content, is_call_self):
            self.component = component
            self.args = args
            self.base_component = base_component
            self.content = content
            self.is_call_self = is_call_self
            self.content_args = None
        
            # set up an indirect filter function that is passed to output
            # buffers, etc.  allows the filtering function can be dynamically 
            # reset to a no-op, in the case that the component calls call_self
            # or cache_self, which should disable the final output filtering
            if isinstance(component, myghty.component.Component) and component.has_filter():
                def filt(f):
                    return component.filter(f, m = request, ARGS = args, **request.global_args)
                self._filter = filt
                self.filter = lambda f: self._filter(f)
        
            
        def reset_filter(self):
            self._filter = lambda f: f

    def out(self, string):
        """a synonym for write"""
        self.write(string)
    
    def write(self, string):
        """writes textual content to the current output buffer."""
        self.buffer.write(string)
        
    def flush_buffer(self):
        """flushes the current output buffer to its destination."""
        self.buffer.flush()

    def clear_buffer(self):
        """clears all content from the current output buffer."""
        self.buffer.clear()

    def execute(self):
        """executes this request after constructed.  This is the first method called,
        and can only be called once per request."""
        if self.executed:
            self._raise_error("Can only call execute() once per request")
        self.executed = True
        if reqinstance.exists():
            existing_request = reqinstance.get()
        else:
            existing_request = None
        reqinstance.put(self)

        # Divert python warnings to log file
        saved_showwarning = warnings.showwarning
        warnings.showwarning = self.__showwarning
        
        if self.parent_request is None \
               or isinstance(self.request_impl, CapturingRequestImpl):
            # top-level request (or subrequest with output captured to
            # out_buffer): set up buffer
            if self.disable_unicode:
                self.buffer = StrRequestBuffer(self.request_impl.buffer)
            else:
                self.buffer = UnicodeRequestBuffer(self.request_impl.buffer,
                                                   self.output_encoding,
                                                   self.encoding_errors)
        else:
            # use parent's request buffer
            self.buffer = self.parent_request.buffer

        self.out = self.write = self.buffer.write # optimization
        
        self.request_args = self.request_impl.request_args
        self.global_args = self.request_impl.global_args
        [self.global_args.setdefault(k, v) for k,v in self.interpreter.global_args.iteritems()]
        [self.global_args.setdefault(k, None) for k in self.interpreter.compiler().allow_globals]
        component = self.component

        # current request-clearing try/finally
        try:
            # exception handling try/except
            try:
                if type(component) == types.StringType:
                    component = self.fetch_lead_component(component)
                elif component.is_file_component():
                    self.request_path = component.path

                self.request_component = component

                depth = 0
                self.wrapper_chain = []
                while (component):
                    self.wrapper_chain.append(component)
                    if not component.is_file:
                        break
                    component = component.parent_component
                    depth += 1
                    if depth >= self.max_recursion:
                        self._raise_error("Max %d levels deep in determining inheritance chain (recursive inheritance pattern?)" % depth);

                if self.disable_wrapping:
                    first_component = self.wrapper_chain[0]
                else:
                    first_component = self.wrapper_chain[-1]

                result = None
                try:
                    result = self.execute_component(first_component, base_component = self.request_component, args = self.request_args)
                except exception.Abort: raise
                except exception.Decline, d: result = d.declined_value
                except exception.AbortRequest: pass

                return result

            except exception.Redirected, e:
                self.request_impl.send_redirect(e.path)
            except exception.Abort, e:
                self.request_impl.send_abort(e.aborted_value, e.reason)
            except exception.Error, e:
                error = e
            except Exception, e:
                error = exception.Error(wrapped = e)
            except:
                e = sys.exc_info()[0]
                error = exception.Error(wrapped = e)

            if error:
                error.initTraceback(self.interpreter)
                if self.parent_request is not None:
                    raise error
                else:
                    if self.raise_error:
                        raise error
                    else:
                        self.request_impl.handle_error(error, self)

        finally:
            # before we leave the execute method, remove all 
            # buffers and restore the request threadlocal
            del self.write, self.out
            self.buffer = None
            warnings.showwarning = saved_showwarning
            
            if existing_request:
                reqinstance.put(existing_request)
            else:
                reqinstance.remove()


    def comp(self, component, **params):
        """component calling method which takes a simple parameter 
        list.
        
        this method is meant to be the default method to call when
        calling components programmatically.
        compare to scomp()."""
        
        return self.execute_component(component = component, args = params)

    def scomp(self, component, **params):
        """component calling method which returns output as a string"""
        self.buffer.push_capture_buffer()
        self.execute_component(component = component, args = params, store = None)
        return self.buffer.pop_capture_buffer().getvalue()
    
    def subexec(self, component, **params):
        """creates a subrequest with the given component and request parameters and executes it.
        
        see create_subrequest()."""
        self.make_subrequest(component, **params).execute()
    
                        
    def execute_component(self, component, args = {}, base_component = None, content = None, store = None, is_call_self = False):
        """component calling method which takes parameter list as a
        dictionary, and allows special execution options.
        
        This method is used by component call tags and internally within
        the Request object."""
        path = None
        if type(component) == types.StringType:
            path = component
            component = self.fetch_component(path)
        
        if self.depth >= self.max_recursion:
            self._raise_error("%d levels deep in component stack (infinite recursive call?)" % self.depth)
        
        
        # if base component was set, use that
        # otherwise, figure it out
        if not base_component:
        
        
            # however, for method and file components, we might have to figure
            # the appropriate base class if there is a colon in the path
            if (
                (component.is_method_component() or not component.is_sub_component()) and
                path and 
                not component.is_module_component() and 
                not re.match(r"(?:SELF|PARENT|REQUEST)(?:\:..*)?$", path)):
                
                match = re.match(r"(.*):", path)
                if match:
                    base_component = self.fetch_component(match.group(1))
                else:
                    base_component = component
                    
                if base_component.is_sub_component():
                    base_component = base_component.owner
            
            elif len(self.execution_stack):
                # base_component defaults to that of the top of the execution stack
                base_component = self.execution_stack[-1].base_component

        frame = Request.StackFrame(self, component, args, base_component, content, is_call_self)
        self.execution_stack.append(frame)

        initial_buffer_state = self.buffer.get_state()

        # if they requested to store the output, push that buffer onto the stack
        if store:
            self.buffer.push_capture_buffer(store)

        if component.has_filter():
            self.buffer.push_filter(frame.filter)

        # Figure out if buffering is needed.
        do_auto_flush = component.use_auto_flush()
        if component.has_filter():
            # If filtering, auto_flush defaults to False
            if do_auto_flush is None or self.dont_auto_flush_filters:
                do_auto_flush = False
        # If this is the top-level component, auto_flush defaults to config val
        if do_auto_flush is None and len(self.execution_stack) == 1:
            do_auto_flush = self.auto_flush

        if not do_auto_flush:
            self.buffer.push_buffer()

        discard_buffered_output = True
        try:
            if component.flags.setdefault('use_cache', False):
                retval = value()
                if self.cache_self(retval = retval, **component.flags):
                    discard_buffered_output = False
                    return retval()

            result = component.run(self, **args)
            discard_buffered_output = False
        finally:
            self.execution_stack.pop()
            # Warning: pop_to_state can raise UnicodeError
            self.buffer.pop_to_state(initial_buffer_state,
                                     discard=discard_buffered_output)

        return result

    def fetch_lead_component(self, path):
        """fetches the top level (initial) component to be executed by this request.  Differs from
        fetch_component in that the resolver context is "request" or "subrequest" and the dhandler
        flag is enabled when resolving.   Also does not support method calls. """
        path = re.sub(r"/+", "/", path)

        request_path = path
        
        try:
            resolution = self.interpreter.resolve_component(path, resolver_context = self.resolver_context, enable_dhandler = self.use_dhandlers, declined_components = self.declined_components)
        except exception.ComponentNotFound, cfound:
            raise cfound.create_toplevel()
        
        # set up a pointer to the current component source, in case
        # the initialization of this component wants to do a fetch_component
        csource = resolution.csource
        
        self.current_csource = csource
        self.resolution = resolution
        
        component = self.interpreter.load_component(csource)

        if hasattr(resolution, 'dhandler_path'):
            self.dhandler_path = resolution.dhandler_path
        
        if self.request_path is None:
            self.request_path = request_path
                    
        return component


    def fetch_component(self, path, resolver_context = 'component', **params):
        """
        Given a component path (absolute or relative), returns a component.
        Handles SELF, PARENT, REQUEST, comp:method, relative->absolute
        conversion, MODULE, and local subcomponents.
        """
        
        hascolon = (string.find(path, ':') != -1)
        
        if not hascolon:
            if path == 'SELF':
                return self.base_component
            elif path == 'PARENT':
                comp = self.current_component.parent_component
                if not comp:
                    self._raise_error("PARENT designator used from component with no parent")
                else:
                    return comp
            elif path == 'REQUEST':
                return self.request_component
            elif path == 'MODULE':
                self._raise_error("MODULE designator requires module name")
        
        if self.has_current_component():
            if hascolon:
                (owner_path, argument) = re.split(':', path, 1)
                if owner_path == 'MODULE':
                    return self.fetch_module_component(argument, **params)
                elif owner_path[0]=='@':
                    return self.fetch_module_component(path[1:], **params)
                
                owner_component = self.fetch_component(owner_path, resolver_context = resolver_context, **params)
                method_component = owner_component.locate_inherited_method(argument)
                return method_component         

            if not hascolon:
                subcomp = self.current_component.get_sub_component(path)
                if subcomp: 
                    return subcomp
                    
            # adjust requested path to the path of the current component            
            path = unixpath.join(self.current_component.dir_name, path)
        else:           
            # no current component.  current csource ?
            if self.current_csource is not None:
                path = unixpath.join(self.current_csource.dir_name, path)
        
        return self.load_component(path, resolver_context = resolver_context, **params) 

    def fetch_module_component(self, moduleorpath, classname = None, raise_error = True):
        """fetches a module-based component.  Usually called by fetch_component when the 
        'MODULE' keyword is given as a prefix. """
        module = None
        modulename = None
        
        if type(moduleorpath) == types.StringType:
            if classname is None:
                (modulename, classname) = moduleorpath.split(':')               
            else:
                modulename = moduleorpath
        else:
            module = moduleorpath
            modulename = moduleorpath.__name__
            if classname is None:
                self._raise_error("classname is required with component module")

        key = "module:" + modulename + ":" + classname
        
        if self.comphash.has_key(key):
            return self.comphash[key]
        
        component = self.interpreter.load_module_component(arg = modulename + ":" + classname)
        self.comphash[key] = component
        
        if raise_error and component is None:
            raise exception.ComponentNotFound("Cant locate component %s" % key)

        return component
        
        
    def load_component(self, path, raise_error = True, resolver_context = None, **params):
        """a mirror of interpreter.load() which caches its results in a request-local
        dictionary, thereby bypassing all the filesystem checks that interp does for
        a repeated file-based request."""

        # since different contexts can come in here, cache based on that key as well
        # technically, all the other context-specific stuff, like dhandler, search upwards
        # etc., should be built into the key as well
        key = "%s_%s" % (str(resolver_context), path)
        
        if self.comphash.has_key(key):
            # get the component from the hash...might be None
            component = self.comphash[key]
        else:
            component = self.interpreter.load(path, raise_error = raise_error, resolver_context = resolver_context, **params)
            self.comphash[key] = component

        if raise_error and component is None:
            raise exception.ComponentNotFound("Cant locate component %s" % path)
    
        return component

    def make_subrequest(self, component, **params):
        """creates a subrequest with the given component and request parameters.
        
        see create_subrequest()."""
        return self.create_subrequest(component, request_args = params)
    
    def create_subrequest(self, component, resolver_context = 'subrequest', **params):
        """base subrequest-creation method.  A subrequest is a request that is a child to 
        this one, enabling execution of a component and its full inheritance chain within 
        this request."""

        if type(component) == types.StringType and self.has_current_component():
            component = unixpath.join(self.current_component.dir_name, component)

        params['component'] = component
        params['parent_request'] = self
        if params.get('out_buffer'):
            params['request_impl'] = CapturingRequestImpl(self.request_impl,
                                                          **params)
            params.setdefault('output_encoding', DEFAULT_OUTPUT_ENCODING)
            params.setdefault('encoding_errors', DEFAULT_ENCODING_ERRORS)
        
        request = self.clone(resolver_context = resolver_context, **params)
        request.request_depth = self.request_depth + 1
        if request.request_depth > self.max_recursion:
            raise exception.Error("recursion limit exceeded")
        return request

    def cache(self, **params):
        """a synonym for get_cache()."""
        return self.get_cache(**params)
        
    def get_cache(self, component = None, **params):
        """returns the given component's cache.  **params is a dictionary of options
        used as the default options for individually cached elements."""
        if component is None:
            component = self.current_comp()
        elif type(component) == types.StringType:
            component = self.fetch_component(component)

        if not self.compcaches.has_key(component.id):
            self.compcaches[component.id] = self.interpreter.cache_args.get_cache(component = component, starttime = component.creationtime, **params)
            
        return self.compcaches[component.id]

    
    def cache_self(self, key = '_self', component = None, retval = None, **params):
        """caches this component's output.  this is called in a "reentrant" fashion; if 
        it returns False, component execution should continue.  If it returns True,
        component execution should cease."""
        cache = self.get_cache(component = component, **params)
        
        def getself():
            rv = value()
            buf = StringIO.StringIO()
            ret = self.call_self(buf, rv)
            if not ret: raise CreationAbortedError()
            else: return (buf, rv())
            
        try:    
            (buf, ret) = cache.get_value(key, cache_createfunc = getself)
            if retval is not None:
                retval.assign(ret)

            # turn off output filtering for the currently
            # executing stack frame
            self.execution_stack[-1].reset_filter()

            self.buffer.write(buf.getvalue())
            return True
        except CreationAbortedError:
            return False

    def call_self(self, output_buffer, return_buffer):
        """calls this component and places its output and return value in the given buffers.
        This component is called in a "reentrant" fashion, where a return value of False
        means to continue component execution and a return value of True means to halt execution
        and return. """
        if self.execution_stack[-1].is_call_self: return False
        
        # turn off output filtering for the currently
        # executing stack frame
        self.execution_stack[-1].reset_filter()
        
        result = self.execute_component(self.current_component, args = self.component_args, store = output_buffer, is_call_self = True)
        return_buffer.assign(result)
        
        return True


    def get_session(self, *args, **params):
        """returns the Session object used by this request.  If the session has not been created
        it will be created when this method is called.  Once the session is created, this method
        will return the same session object repeatedly.  **params is a dictionary of options used
        when the session is first constructed; if it already exists, the parameters are ignored."""
        return self.request_impl.get_session(*args, **params)
        
    
    def has_current_component(self):
        """returns if this request has a currently executing component.
        
        This could return false if the request's top-level-component is in the loading
        stage."""
        return len(self.execution_stack) > 0
        
    def component_exists(self, path, **params):
        """returns True if the given component path exists."""
        return self.interpreter.component_exists(path, **params)

    def apply_escapes(self, text, escapes):
        """applies the given escape flags to the given text.  escapes is a list of
        escape flags, such as 'h', 'u', and 'x'."""
        # XXX: If you change this string coercion code, you should also
        #      make matching changes to {Unicode,Str}RequestBuffer.write()
        if text is None:
            text = ''
        elif self.disable_unicode:
            text = str(text)
        else:
            text = unicode(text)
        
        esctable = self.interpreter.escapes
        try:
            escfuncs = [ esctable[esc] for esc in escapes ]
        except KeyError, k:
            self._raise_error("no such escape '%s'" % k)

        for f in escfuncs:
            text = f(text)
        return text
    

    def _get_status(self):
        """debugging method that returns the current execution stack entry"""
        return ("Base Comp: %s\n_request Comp: %s\n_current Comp: %s\n" %
            (self.base_component.id, 
                self.request_component.id, 
                self.current_component.id()))
            
    
    def abort(self, status_code=None, reason=None):
        """raises an abort exception.  """
        if status_code is None:
            raise exception.AbortRequest()
        else:
            raise exception.Abort(aborted_value=status_code, reason=reason)

    def decline(self, returnval = None):
        """used by dhandlers to decline handling the current request.  Control will
        be passed to the next enclosing dhandler, or if none is found a ComponentNotFound
        (and possibly 404 error) is raised."""
        self.buffer.clear()
        subreq = self.make_subrequest(component=self.request_path,
            request_args = self.request_args)
        subreq.declined_components[self.request_component.id] = self.request_component
        ret = subreq.execute()
        raise exception.Decline(declined_value = ret)
    
    def callers(self, index = None):
        """returns the list of components in the current call stack.  if an integer 
        index is given, returns the component at that position in the current call stack."""
        if index is None:
            return map(lambda f: f.component, self.execution_stack)         
        else:
            return self.execution_stack[index].component
            
    def caller_args(self, index = None):
        """returns the list of component arguments in the current call stack.  if an 
        integer index is given, returns the component arguments at that position in the
        current call stack."""
        if index is None:
            return map(lambda f: f.args, self.execution_stack)          
        else:   
            return self.execution_stack[index].args
    
    def call_stack(self, index = None):
        """returns the current execution stack, which consists of StackFrame objects.
        if an integer index is given, returns the StackFrame object at that position
        in the current call stack."""
        if index is None:
            return self.execution_stack
        else:
            return self.execution_stack[index]
    
    def call_next(self, **params):
        """used within an inheritance chain to call the next component in the chain.  If 
        **params are given, each parameter will override the arguments sent to this component
        when calling the next component."""
        comp = self.fetch_next()
        args = {}
        args.update(self.request_args)
        args.update(params)
        
        return self.comp(comp, **args)

    def fetch_next(self):
        """in an inheritance chain (i.e. of autohandlers), returns the next component in 
        the chain"""
        try:
            self.wrapper_chain.pop()
            return self.wrapper_chain[-1]
        except IndexError:
            self._raise_error("No component available for fetch_next()")

    def fetch_all(self):
        """pops off the entire remaining list of components in the inheritance chain."""
        ret = []
        while len(self.wrapper_chain):
            ret.append(self.wrapper_chain.pop())
        return ret
        
    def send_redirect(self, path, hard=True):
        """sends a redirect to the given path.  If hard is True, sends an HTTP 302 redirect
        via the underlying RequestImpl being used.  If False, clears out the current output 
        buffer and executes a subrequest with the given path.  The path can also contain 
        url encoded query string arguments with a question mark which will be converted
        to key/value arguments for the next request, even if hard=False."""
        if hard:
            raise exception.Redirected(path)
        else:
            if re.search(r'\?', path):
                (path, query) = path.split('?')
                args = dict([(escapes.url_unescape(m.group(1)), escapes.url_unescape(m.group(2))) for m in re.finditer(r'([^=&]*)=([^=&]*)', query)])
            else:
                args = {}

            self.buffer.clear()
            req = self.create_subrequest(path, resolver_context = self.resolver_context, request_args = args)
            req.execute()
            self.buffer.flush()
            raise exception.AbortRequest()
        
    def has_content(self):
        """returns whether or not the current component call was called
        with captured content specified"""
        return self.execution_stack[-1].content != None

    def call_content(self, **kwargs):
        """calls the "captured content" function within a component call with content"""
        if not self.execution_stack[-1].content:
            return

        frame = self.execution_stack.pop()
        try:
            self.execution_stack[-1].content_args = kwargs
            frame.content()
        finally:
            self.execution_stack[-1].content_args = None
            self.execution_stack.append(frame)
        
    def content(self, **kwargs):
        """when inside a component call with content, this method calls the
        "captured content" function and buffers the content, returning the string
        value."""

        self.buffer.push_capture_buffer()
        try:
            self.call_content(**kwargs)
        finally:
            buffer = self.buffer.pop_capture_buffer()
        return buffer.getvalue()
        
    def closure_with_content(self, func, content=None, args=None):
        """given a function, will execute it with the given arguments, after pushing
        the given content function onto the call stack via a new StackFrame, enabling
        the function to call it via the content() method."""
        frame = Request.StackFrame(self, func, args, self.base_component, content, False)
        if args is None:
            args = {}
        self.execution_stack.append(frame)
        try:
            func(**args)
        finally:
            self.execution_stack.pop()
        
    def _raise_error(self, error):
        self.interpreter.raise_error(error)

    def log(self, message):
        """writes a message to the request log.  The log is RequestImpl-specific and can be 
        standard error, a webserver error log, or a user-defined file object."""
        self.request_impl.log(message)

    def __showwarning(self, message, category, filename, lineno):
        """A custom warnings handler.
        """
        interpreter = self.interpreter
        try:
            # Try to map (filename, lineno) back to original
            try:
                comprec = interpreter.reverse_lookup[filename]
            except KeyError:
                comprec = interpreter.reverse_lookup['memory:' + filename]
            reversefile = exception.Error.ReverseFile(interpreter, comprec)
            csource = comprec.csource
            filename, lineno = \
                      csource.file_path, reversefile.get_line_number(lineno)
        except:
            pass
        mesg = warnings.formatwarning(message, category, filename, lineno)
        if mesg.endswith('\n'):
            mesg = mesg[:-1]
        self.log(mesg)

    # property accessors
    
    logger = property(lambda self: _Logger(self), doc="""\
    returns the logger for this Request, which is a file-like object.
    """)
    
    depth = property(lambda self: len(self.execution_stack), doc="""\
    Returns the current depth of the execution stack.
    """)

    base_component = property(lambda self: self.execution_stack[-1].base_component, doc="""\
    Returns the "base component" for the current stack frame, which is either the top level component
    or the owning component of a method component.
    
    """)

    component_args = property(lambda self: self.execution_stack[-1].args, doc="""\
    returns the argument dictionary from the current stack frame, corresponding to the arguments
    sent to the currently executing component.
    """)
    
    content_args = property(lambda self:self.execution_stack[-1].content_args, doc="""\
    returns the component-call-with-content argument dictionary from the current stack frame, 
    corresponding to the arguments sent in an m.content() call.
    """)
    
    caller = property(lambda self: self.execution_stack[-2].component, doc="""\
    returns the calling component for the currently executing component.
    """)
    
    dhandler_argument = property(lambda self: self.dhandler_path, doc="""\
    returns the current dhandler_argument, which corresopnds to the remaining path tokens
    in the request URI when executing a dhandler.
    """)
    
    current_component = property(lambda self: self.execution_stack[-1].component, doc="""\
    returns the component currently executing from the current stack frame.
    """)


    def get_output_encoding(self):
        if self.buffer:
            return self.buffer.encoding
        else:
            return self.__output_encoding[0]
    output_encoding = property(
        get_output_encoding,
        doc="""The ouput encoding for the request

        This is the encoding of the output written to the output buffer
        of the top-level request.  (I.e. the encoding delivered to the
        user.)

        This is a read-only attribute, though you can change its value
        using the `set_output_encoding`_ method.
        """)

    def get_encoding_errors(self):
        if self.buffer:
            return self.buffer.errors
        else:
            return self.__output_encoding[1]
    encoding_errors = property(
        get_encoding_errors,
        doc="""The encoding error handling strategy for the request

        This is the error handling strategy used when encoding text
        writtent to the output buffer of the top-level request.

        This is a read-only attribute, though you can change its value
        using the `set_output_encoding`_ method.
        """)

    def set_output_encoding(self, encoding, errors='strict'):
        """Change the output_encoding.
        
        Note that in most cases you do not want to change it after you
        have written any output (as then your output will be in two
        different encodings --- probably not what you wanted unless,
        perhaps, you are generating Mime multipart output.)
        """
        if self.buffer:
            self.buffer.set_encoding(encoding, errors)
            self.__output_encoding = self.buffer.encoding, self.buffer.errors
        else:
            self.__output_encoding = encoding, errors
    
    # deprecated getter methods !   

    def get_attribute(self, key): 
        """deprecated. use the attributes property."""
        return self.notes(key)
    def set_attribute(self, key, value): 
        """deprecated. use the attributes property."""
        self.notes(key, value)
    def get_attributes(self): 
        """deprecated. use the attributes property."""
        return self.notes
    def get_depth(self): 
        """deprecated.  use the depth property."""
        return self.depth
    def get_log(self): 
        """deprecated.  use the logger property."""
        return self.logger
    def get_start_time(self): 
        """deprecated. use the starttime property."""
        return self.starttime
    def get_request_args(self):
        """deprecated.  use the request_args property."""
        return self.request_args 
    def get_base_component(self): 
        """deprecated.  use the base_component property."""
        return self.base_component
    def base_comp(self): 
        """deprecated.  use the base_component property."""
        return self.base_component
    def get_request_component(self): 
        """deprecated.  use the request_component property."""
        return self.request_component
    def request_comp(self): 
        """deprecated.  use the request_component property."""
        return self.request_component
    def get_component_args(self):
        """deprecated.  use the component_args property."""
        return self.component_args
    def get_dhandler_argument(self): 
        """deprecated.  use the request_path or dhandler_argument property."""
        return self.request_path
    def dhandler_arg(self): 
        """deprecated.  use the request_path or dhandler_argument property."""
        return self.request_path
    def get_request_path(self): 
        """deprecated.  use the request_path property."""
        return self.request_path
    def get_interpreter(self): 
        """deprecated.  use the interpreter property."""
        return self.interpreter
    def get_current_component(self): 
        """deprecated.  use the current_component property."""
        return self.current_component
    def current_comp(self): 
        """deprecated.  use the current_component property."""
        return self.current_component
    

def instance():
    """returns the Request instance corresponding to the current thread"""
    return reqinstance.get()

def threadlocal(value = None):
    """returns a thread local container, with initial value that of the given variable"""
    return ThreadLocal(value)

def value(value = None):
    """returns a value container object.  useful for mutating data that was instantiated
    in a requestonce or shared block"""
    return Value(value)
    
class AbstractRequestImpl(object):
    def clone(self):raise NotImplementedError()
    def handle_error(self, e, m, **params):raise NotImplementedError()
    def get_session(self, **params):raise NotImplementedError()

    def send_redirect(self, path):raise NotImplementedError()
    def send_abort(self, ret, reason):raise NotImplementedError()
    def log(self, message):raise NotImplementedError()

    def _run_error_handler(self, handler, logger, error, m, **params):
        err = None
        try:
            if handler:
                return handler(error, m, **params)
        except exception.Error, e:
            err = e
        except Exception, e:
            err = exception.Error(wrapped = e)
        except:
            e = sys.exc_info()[0]
            err = exception.Error(wrapped = e)

        if err:
            err.initTraceback(m.interpreter)
            logger.write("Custom error handler had an error:")
            logger.writelines(string.split(err.format(), "\n"))

        return False

    def __init__(self):
        self.global_args = None
        self.request_args = None
        self.logger = None
        self.buffer = None
        self.request = None

class DefaultRequestImpl(AbstractRequestImpl):

    def __init__(self, out_buffer = None, global_args = None, request_args = None, error_handler = None, **params):
        if out_buffer:
            self.out_buffer = out_buffer
        else:
            self.out_buffer = sys.__stdout__

        self.error_handler = error_handler
        
        if request_args is None:
            request_args = {}
        self.request_args = request_args
        if global_args is None:
            self.global_args = {}
        else:
            # copy the global_args in case they are synonymous with those given to the
            # interpreter
            self.global_args = global_args.copy()
        self.logger = sys.stderr
        
        self.buffer = self.out_buffer
            
    def handle_error(self, error, m, **params):
        if self._run_error_handler(self.error_handler, self.logger, error, m, **params): return
        
        sys.stderr.write(error.format())

    def log(self, message):self.logger.write(message + "\n")

    def clone(self, **params):
        cloner = ConstructorClone(self, **params)
        return cloner.clone()

class _Logger:
    '''A file-like object which "writes" to an object ``log()`` method.
    '''
    def __init__(self, impl): self.impl = impl
    def write(self, text): self.impl.log(text)
    def writelines(self, lines):
        for text in lines: self.write(text)

class CapturingRequestImpl(AbstractRequestImpl):
            
    def __init__(self, parent_request_impl, out_buffer,
                 request_args=None,
                 global_args=None,
                 error_handler=None,
                 **params):
        self.buffer = self.out_buffer = out_buffer
        self.parent_request_impl = parent_request_impl

        if request_args is None:
            request_args = parent_request_impl.request_args.copy()
        self.request_args = request_args

        # XXX: No .copy() here, subrequest can change parent requests
        # global_args.  Is that correct?
        if global_args is None:
            global_args = parent_request_impl.global_args
        self.global_args = global_args

        self.error_handler = error_handler

    logger = property(lambda self: _Logger(self))
    
    def handle_error(self, error, m, **params):
        if self.error_handler:
            self._run_error_handler(self.error_handler,
                                    self.logger, error, m, **params)
        else:
            self.parent_request_impl.handle_error(error, m, **params)
        
    def log(self, message):
        self.parent_request_impl.log(message)
    



import posixpath as unixpath
import myghty.component as component
import myghty.escapes as escapes
import re, os
import shoppingdata, shoppingmodel, form
from statemachine import *
import highlight

# shopping cart commands
CMD_ADD = 'add'
CMD_UPDATE = 'update'
CMD_REMOVE = 'remove'
CMD_NEXT = 'next'
CMD_PREVIOUS = 'previous'

# checkout process states
# the numbers are so they can be compared for ordering
CHECKOUT_START = '1_start'
CHECKOUT_BILLING = '2_billing'
CHECKOUT_SHIPPING = '3_shipping'
CHECKOUT_PAYMENT = '4_payment'
CHECKOUT_CONFIRM = '5_confirm'
CHECKOUT_DONE = '6_done'


## module components ##

class _Store(object):

    def do_component_init(self, component):
        self.store_path = component.interpreter.attributes.get('store_path')

    def catalog(self, m):
        """catalog component, parses the path and displays product list pages"""
        app = _RequestHelper(m, self)
        match = re.match(r".*/catalog/(.*/)?$", m.request_path)
        if not match:
            m.abort(404)

        path = match.group(1)
        if path is not None:
            path = escapes.url_unescape(path)

        category = shoppingdata.store

        if path:
            try:
                category = category.get_category('/' + path)
            except KeyError:
                m.abort(404)

        if len(category.items) > 0:
            items = category.items
        else:
            items = shoppingdata.featureditems.items

        app.show_template("catalog.myt", category = category, items = items)

    def item(self, m):
        """item component, parses the path and displays individual item pages"""
        match = re.match(r".*/item/(.*)/?$", m.request_path)
        itemname = match.group(1)

        if itemname is not None:
            itemname = escapes.url_unescape(itemname)

        try:
            item = shoppingdata.store.get_item(itemname)
        except KeyError:
            m.abort(404)

        _RequestHelper(m, self).show_template("item.myt", category = item.primarycategory, item = item)

    def cart(self, m, ARGS, cmd = None, qty = None, itemname = None, index = None):    
        """cart component, displays the shopping cart and handles updates"""
        app = _RequestHelper(m, self)
        save = False
        user = app.get_user()
        if user.cart is None:
            user.cart = shoppingmodel.Cart()
            save = True

        if itemname is not None:
            item = shoppingdata.store.get_item(itemname)
            variants = []
            for category in item.get_variant_categories():
                variants.append(item.get_variant(category, ARGS['variant_' + category]))

        if cmd == CMD_ADD:
            try:
                user.cart.add_item(item, int(qty), variants)
                save = True
            except ValueError:
                pass
        elif cmd == CMD_UPDATE:
            for i in range(0, len(user.cart.items)):
                try:
                    user.cart.items[i].quantity = int(ARGS['qty_' + str(i)])
                except ValueError:
                    pass
            save = True
        elif cmd == CMD_REMOVE:
            try:
                user.cart.remove_item(int(index))
                save = True
            except ValueError:
                pass

        if save:
            app.save_session()

        app.show_template("cart.myt", cart=user.cart)

    def checkout(self, m, ARGS, cmd = None, **params):
        """manages the checkout process, including form field handling and state transition.
        Delegates most of the work onto a CheckoutTransitions object which handles state transitions."""
        app = _RequestHelper(m, self)
        smachine = _CheckoutTransitions(app)
        smachine.do_transition(cmd)

        if smachine.template is not None:
            app.save_session()
            app.show_template(smachine.template, ck_state = smachine.state, form = smachine.form, invoice = smachine.invoice)


    def source(self, m, r):            
        """view source component, opens the source file up and displays.
        clearly, one should be careful with the configuration of this type of component 
        lest it be too flexible in what it shows.
        """

        filename = r.filename

        if os.path.isdir(filename):
            m.abort(403)

        try:
            f = file(filename)
        except IOError:
            m.abort(404)

        r.content_type = 'text/html'        
        s = f.read()
        s = highlight.highlight(s, filename = filename)

        _RequestHelper(m, self).show_template("viewsource.myt", name = m.request_path, source = s)


index = _Store()


class _RequestHelper(object):
    """a per-request helper object that performs common functions using the request object as well
    as the session."""
    def __init__(self, m, handler):
        self.m = m
        self.handler = handler

        session = m.get_session()

        if not session.has_key('shopping_user'):
            user = shoppingmodel.User()
            session['shopping_user'] = user
            session.save()

    def get_user(self):
        return self.m.get_session()['shopping_user']

    def save_session(self):
        self.m.get_session().save()

    def show_template(self, template, **params):
        self.m.subexec(self.handler.store_path + '/' + template, **params)



class _CheckoutTransitions(StateMachine):
    """
    a StateMachine implementation storing all the possible transitions for the shopping cart.
    
    the state itself is stored in the session and is also matched against a hidden form variable.
    if the two don't match, no transition occurs.
    
    this approach insures that the state of the checkout is determined by the server and
    cannot be overridden by a fabricated HTTP request.  It also renders any errors 
    related to the "reload" button irrelevant.  users can go 
    back/next/reload till the sun comes down on any checkout page and
    it wont screw up their state/submit twice/etc.
    """
    
    def __init__(self, app):
        self.app = app
        self.ARGS = app.m.request_args
        self.user = app.get_user()
        self.invoice = None            
        self.session = app.m.get_session()

    def create_form(self):
        """returns a Form object representing all the fields we are going to collect.
        the Form is stored in the users session."""
        
        return form.Form('checkout',
        [
            form.FormField('ck_state', default=CHECKOUT_START),
            form.FormField('currentform'),
            
            form.SubForm('billing', [
                form.IntFormField('useaddress', options = []),
                form.FormField('firstname', description = "First Name", required=True),
                form.FormField('lastname', description = "Last Name", required=True),
                form.FormField('street1', description = "Street", required=True),
                form.FormField('street2', description = "Street"),
                form.FormField('city', description = "City", required=True),
                form.FormField('state', description = "State", required=True),
                form.FormField('zipcode', description = "Zip Code", required=True),
                form.FormField('country', descriptinon = "Country", required=True, default='USA'),
            ]),
            
            form.SubForm('shipping', [
                form.IntFormField('useaddress', options = []),
                form.FormField('firstname', description = "First Name", required=True),
                form.FormField('lastname', description = "Last Name", required=True),
                form.FormField('street1', description = "Street", required=True),
                form.FormField('street2', description = "Street"),
                form.FormField('city', description = "City", required=True),
                form.FormField('state', description = "State", required=True),
                form.FormField('zipcode', description = "Zip Code", required=True),
                form.FormField('country', descriptinon = "Country", required=True, default='USA'),
                
            ]),
            
            form.SubForm('payment', [
                form.FormField('ccname', description="Credit Card Name", required = True, textsize=40),
                form.FormField('cctype', description = "Credit Card Type", required = True, options=(
                                        ('amex', 'American Express'),
                                        ('visa', 'Visa'),
                                        ('mastercard', 'Master Card'),
                                        )),
                form.CCFormField('ccnumber', description ="Credit Card Number", required = True, textsize=20, default = '371449635398431'),
                form.DateFormField('ccexp', description ="Credit Card Expiration Date", required = True, fields = ['month', 'year'], yeardeltas = range(0, 9)),
            ]),
        ])


    def do_transition(self, transition):

        if not self.session.has_key('invoice_form') or transition is None:
            self.state = CHECKOUT_START
            self.invoice_form = self.create_form()
            self.session['invoice_form'] = self.invoice_form
        else:
            self.invoice_form = self.session['invoice_form']
            self.state = self.invoice_form.elements['ck_state'].displayvalue
            self.set_current_form(self.invoice_form.elements['currentform'].displayvalue)
            
            if self.state >= CHECKOUT_CONFIRM:
                self.template = 'confirm.myt'
                self.invoice = self.user.invoice
            else:
                self.template = 'checkout.myt'
            
            if self.state != self.ARGS['ck_state'] or self.state == CHECKOUT_DONE:
                return
        
        if transition is None:
            transition = CMD_NEXT
        
        try:
            self.state = StateMachine.do_transition(self, self.state, transition)
            self.invoice_form.elements['ck_state'].set_value(self.state)
        except AbortTransition: pass

    def set_current_form(self, name):
        self.invoice_form.elements['currentform'].set_value(name)
        self.form = self.invoice_form.elements[name]

    def set_address_dropdowns(self):
        addresses = [(i, self.user.addresses[i].street1) for i in range(0, len(self.user.addresses))]

        if len(addresses):
            addresses.insert(0, ('', 'select an address'))

            self.invoice_form.elements['billing'].elements['useaddress'].options = addresses
            self.invoice_form.elements['shipping'].elements['useaddress'].options = addresses
        
    def extract_address(self):
        address = shoppingmodel.Address()
        self.form.reflect_to(address)
        for a in self.user.addresses:
            if address == a: break
        else:
            self.user.addresses.append(address)
        self.set_address_dropdowns()
        return address

    def fill_address(self):
        address = self.user.addresses[self.form.elements['useaddress'].currentvalue]
        self.form.elements['useaddress'].set_value(None)
        self.form.reflect_from(address)
        self.form.unvalidate()
        return address
        
    def start_to_billing(self):
        self.set_address_dropdowns()
        self.set_current_form('billing')
        self.template = 'checkout.myt'
        
    def billing_to_start(self):
        self.app.show_template("cart/")    
        self.template = None

    def billing_to_shipping(self):
        self.form.set_request(self.ARGS)

        if self.form.elements['useaddress'].currentvalue is not None:
            address = self.fill_address()
            self.set_current_form('shipping')
            self.form.unvalidate()

        elif self.form.is_valid():
            self.extract_address()
            self.set_current_form('shipping')
            self.form.unvalidate()
        else:
            raise AbortTransition()

    def shipping_to_billing(self):
        self.set_current_form('billing')
        self.form.unvalidate()

    def shipping_to_payment(self):
        self.form.set_request(self.ARGS)

        if self.form.elements['useaddress'].currentvalue is not None: 
            address = self.fill_address()
            ccname = address.firstname + " " + address.lastname
            
            self.set_current_form('payment')
            self.form.elements['ccname'].set_value(ccname)
            self.form.unvalidate()
        elif self.form.is_valid():
            address = self.extract_address()
            ccname = address.firstname + " " + address.lastname
            
            self.set_current_form('payment')
            self.form.elements['ccname'].set_value(ccname)
            self.form.unvalidate()
        else:
            raise AbortTransition()
            
        
    def payment_to_shipping(self):
        self.set_current_form('shipping')
        self.form.unvalidate()

    def payment_to_confirm(self):
        self.form.set_request(self.ARGS)

        if self.form.is_valid():
            billing= shoppingmodel.Address()
            self.invoice_form.elements['billing'].reflect_to(billing)

            shipping = shoppingmodel.Address()
            self.invoice_form.elements['shipping'].reflect_to(shipping)

            cc = shoppingmodel.CreditCard()
            self.invoice_form.elements['payment'].reflect_to(cc)

            invoice = shoppingmodel.Invoice(self.user.cart.items, self.user, billing, shipping, cc)

            self.user.invoice = invoice
            self.invoice = invoice
            
            self.template = 'confirm.myt'
        else:
            raise AbortTransition()

    def confirm_to_payment(self):
        self.set_current_form('payment')
        self.template = 'checkout.myt'
        self.form.unvalidate()
        
    def confirm_to_done(self):
        self.invoice = self.user.invoice
        self.user.cart.items = []
        self.template = 'confirm.myt'
    
    transitions = dict([
        Transition(CHECKOUT_START, CHECKOUT_BILLING, CMD_NEXT, start_to_billing),
        Transition(CHECKOUT_BILLING, CHECKOUT_START, CMD_PREVIOUS, billing_to_start),
        Transition(CHECKOUT_BILLING, CHECKOUT_SHIPPING, CMD_NEXT, billing_to_shipping),
        Transition(CHECKOUT_SHIPPING, CHECKOUT_BILLING, CMD_PREVIOUS, shipping_to_billing),
        Transition(CHECKOUT_SHIPPING, CHECKOUT_PAYMENT, CMD_NEXT, shipping_to_payment),
        Transition(CHECKOUT_PAYMENT, CHECKOUT_SHIPPING, CMD_PREVIOUS, payment_to_shipping),
        Transition(CHECKOUT_PAYMENT, CHECKOUT_CONFIRM, CMD_NEXT, payment_to_confirm),
        Transition(CHECKOUT_CONFIRM, CHECKOUT_PAYMENT, CMD_PREVIOUS, confirm_to_payment),
        Transition(CHECKOUT_CONFIRM, CHECKOUT_DONE, CMD_NEXT, confirm_to_done),
    ])
        







# encoding: latin1
# test_html_errors.py
"""Test HTML formatting of errors.
"""
import re, unittest
import testbase
from myghty.exception import Error, Syntax

class HTMLDecodingErrorBase(object):

    expected_error = Error
    wrapped_error = None
    expected_html = "euro:&euro;"
    
    def _testComponent(self, component_maker):
        try:
            self.runComponent(component_maker(self.csource))
        except Error, ex:
            if self.expected_error:
                self.failUnless(isinstance(ex, self.expected_error))
            if self.wrapped_error:
                self.failUnless(isinstance(ex.wrapped, self.wrapped_error))
        else:
            self.fail()

        if self.expected_html not in ex.htmlformat():
            print ex.htmlformat()
        self.failUnless(self.expected_html in ex.htmlformat())
        
    def testFileComponent(self):
        self._testComponent(self.makeFileBasedComponent)

    def testMemoryComponent(self):
        self._testComponent(self.makeMemoryComponent)

class PythonErrorInComponentTests(HTMLDecodingErrorBase,
                                  testbase.ComponentTester):
    csource = """# encoding: iso-8859-15
                 % m.setTYPO_output_encoding('UTF-8')
                 euro:\xa4
                 <br />encoding is: <% m.output_encoding %>
                 """

    wrapped_error = AttributeError

class PythonErrorInComponentCodeline(HTMLDecodingErrorBase,
                                  testbase.ComponentTester):
    csource = """# encoding: iso-8859-15
                 % u'euro:\xa4' + 1
                 """

    wrapped_error = TypeError
   
class SyntaxErrorInComponentTests(HTMLDecodingErrorBase,
                                  testbase.ComponentTester):
    
    csource = """# encoding: iso-8859-15
                 <%python scope='bogus'>
                 </%python>
                 euro:\xa4
                 """

    expected_error = Syntax

class ErrorInModuleComponent(HTMLDecodingErrorBase,
                             testbase.ComponentTester):
    csource = """# encoding: iso-8859-15
                 def f(m):
                     return u'euro:\xa4' + 1
                 """

    def testFileComponent(self):
        def comp_maker(csource):
            module = self.makeModule(csource)
            return module.f
        self._testComponent(comp_maker)

    def testMemoryComponent(self):
        pass


class AnonymousPythonErrorInComponentTests(HTMLDecodingErrorBase,
                                           testbase.ComponentTester):
    csource = """# encoding: iso-8859-15
                 <%python>
                 g = dict()
                 eval(compile('# encoding: iso-8859-15\\n'
                              'def f(): return u\"euro:\xa4\" + 1\\n',
                              '<memory>', 'exec', 0, True), g, g)
                 f = g['f']
                 </%python>
                 % f()
                 """

    wrapped_error = TypeError

    expected_html = "2: None"


################################################################        

import codecs, StringIO
from myghty.exception import _parse_encoding as parse_encoding

class parse_encoding_Tests(testbase.MyghtyTest):
    def testParseEncoding(self, src="", coding=None):
        f = StringIO.StringIO(src)
        try:
            self.failUnlessEqual(parse_encoding(f), coding)
        finally:
            self.failUnlessEqual(f.read(), src) # test that file is rewound
        
    def testSimple(self):
        self.testParseEncoding("# -*- coding: foo -*-\n", 'foo')
        self.testParseEncoding("# First line \n"
                               "# vim:fileencoding=bar\n",
                               'bar')
        self.testParseEncoding("# First line\n"
                               "# Second line\n"
                               "# -*- coding: foo -*-\n",
                               None)

    def testBom(self):
        self.testParseEncoding(codecs.BOM_UTF8 + "# Howdy\n",
                               'utf_8')

    def testBadBom(self):
        self.failUnlessRaises(SyntaxError,
                              self.testParseEncoding,
                              codecs.BOM_UTF8 + "# coding: ascii\n",
                              'utf_8')

        self.failUnlessRaises(SyntaxError,
                              self.testParseEncoding,
                              codecs.BOM_UTF8 + "'line1'\n# coding: ascii\n",
                              'utf_8')

    def testEsoteric(self):
        self.testParseEncoding('print """\n'
                               '# coding: foo\n'
                               '"""\n',
                               None)

        self.testParseEncoding('(\n'
                               '# coding: foo\n'
                               ')\n',
                               None)


            
if __name__ == '__main__':
    unittest.main()



"""
    a set of classes that represent an HTML form, including its field names, 
    the values of those fields corresponding to their application setting
    as well as the value pulled from an HTTP request, and validation rules
    for the fields.  "hierarchical forms" can be created as well, allowing a form
    to be grouped into "subforms", which may live on the same HTML page or across
    several HTML pages.
"""

from myghty.util import *
import inspect, datetime, types

class FormElement:
    def __init__(self, name, **params):
        self.name = name
        self.description = ''

    def set_form(self, form):pass
    def set_request(self, req, validate = True):pass
    
    def is_valid(self):return None
    def unvalidate(self):pass
    def get_valid_message(self):return ''
    
class Form(FormElement):
    def __init__(self, name, elements, **params):
        FormElement.__init__(self, name)
        
        self.isvalid = None
        self.elements = OrderedDict()
        for elem in elements:
            self.elements[elem.name] = elem
            elem.set_form(self)

    def set_form(self, form):pass

    def show_default(self):[elem.show_default() for elem in self.elements.values()]
    def show_value(self):[elem.show_value() for elem in self.elements.values()]
    def show_request_value(self):[elem.show_request_value() for elem in self.elements.values()]

    def is_valid(self):
        return self.isvalid

    def get_valid_message(self):
        if self.isvalid:
            return "Form is valid"
        elif self.isvalid is False:
            return "Some fields could not be validated"
    
    def unvalidate(self):
        self.isvalid = None
        for elem in self.elements.values(): elem.unvalidate()
    
    def get_field(self, name):
        try:
            return self.elements[name]
        except KeyError:
            return None
        
    def set_request(self, req, validate = True):
        self.isvalid = True
        for elem in self.elements.values():
            elem.set_request(req, validate)
            if not elem.is_valid():
                self.isvalid = False
                
    def _fieldname(self, formfield):
        return formfield.name

    def reflect_from(self, object):
        for elem in self.elements.values():
            elem.reflect_from(object)
        
    def reflect_to(self, object):
        for elem in self.elements.values():
            elem.reflect_to(object)

    
class SubForm(Form):
        
    def _fieldname(self, formfield):
        return self.name + "_" + formfield.name
        
class FormField(FormElement):
    def __init__(self, name, description = None, required = False, default = None, textsize = None, options = None, **params):
        FormElement.__init__(self, name)

        if description is None:
            self.description = name
        else:
            self.description = description
        
        self.required = required
        self.options = options
        self.textsize = textsize

        # default value (any type)
        self.default = default
        
        # programmatically set value (any type)
        self.value = None
        
        # value taken from the request (any type)
        self.request_value = None
        
        # current used value (any type)
        self.currentvalue = default

        # string display value
        if self.currentvalue is None:
            self.displayvalue = ''
        else:
            self.displayvalue = str(self.currentvalue)
        
        
        self.displayname = None
        self.valid_message = ''        
        self.isvalid = None
        
    def set_form(self, form):
        self.displayname = form._fieldname(self)

    def set_value(self, value):
        self.value = value
        self.currentvalue = value
        if self.currentvalue is None:
            self.displayvalue = ''
        else:
            self.displayvalue = str(self.currentvalue)
    
    def show_default(self):self.displayvalue = self.default
    def show_value(self):self.displayvalue = self.value
    def show_request_value(self):self.displayvalue = self.request_value

    def reflect_to(self, object):
        if hasattr(object, self.name):
            setattr(object, self.name, self.displayvalue)

    def reflect_from(self, object):
        if hasattr(object, self.name):
            self.set_value(getattr(object, self.name))

    def is_valid(self):
        """returns whether or not this field is valid.  note that the 
        third state of None indicates this field has not been validated."""
        return self.isvalid

    def get_valid_message(self):
        return self.valid_message
        
    def set_request(self, request, validate = True):
        
        """sets the request for this form.   if validate is True, also 
        validates the value."""

        try:
            self.request_value = request[self.displayname]
        except KeyError:
            self.request_value = None
    
        if validate:
            if self.required and not self.request_value:
                self.isvalid = False
                self.valid_message = 'required field "%s" missing' % self.description
            else:
                self.isvalid = True
        
        if self.request_value is None:
            self.displayvalue = ''
        else:
            self.displayvalue = str(self.request_value)
            
        self.currentvalue = self.request_value
    
    def unvalidate(self):
        """resets the isvalid state of this form to None"""
        self.isvalid = None
        self.valid_message = ''
        

class IntFormField(FormField):
    def __init__(self, *args, **params):
        FormField.__init__(self, *args, **params)
        self.currentvalue = None
        
            
    def set_request(self, request, validate = True):
        FormField.set_request(self, request, validate)
        
        try:
            if self.currentvalue == '':
                self.currentvalue = None
                
            if self.currentvalue is not None:
                self.currentvalue = int(self.currentvalue)
        except ValueError:
            if self.isvalid and validate:
                self.valid_message = 'field "%s" must be an integer number' % self.description
                self.isvalid = False
                self.currentvalue = None
                
                
class CompoundFormField(SubForm):
    """
    a SubForm that acts like a single formfield in that it contains a single value,
    but also contains subfields that comprise elements of that value.
    
    examples: a date with year, month, day fields, corresponding to a date object
    
    more exotic examples: a credit card field with ccnumber, ccexpiration fields corresponding to a 
    CreditCard object, an address field with multiple subfields corresopnding to an Address object, etc.
    
    """

    def __init__(self, name, elements, description = None, **params):
        SubForm.__init__(self, name, elements, **params)
        self.value = None
        self.request_value = None
        self.displayvalue = None

        if description is None:
            self.description = name
        else:
            self.description = description
        
    def set_value(self, value):
        self.value = value
        self.currentvalue = value
        self.displayvalue = str(value)
        self.set_compound_values(value)        

    def reflect_to(self, object):
        if hasattr(object, self.name):
            setattr(object, self.name, self.currentvalue)

    def reflect_from(self, object):
        if hasattr(object, self.name):
            self.set_value(getattr(object, self.name))

    def show_default(self):
        self.displayvalue = self.default
        [elem.show_default() for elem in self.elements.values()]
        
    def show_value(self):
        self.displayvalue = self.value
        [elem.show_value() for elem in self.elements.values()]
        
    def show_request_value(self):
        self.displayvalue = self.request_value
        [elem.show_request_value() for elem in self.elements.values()]
        


class CCFormField(FormField):

    def set_request(self, request, validate = True):
        FormField.set_request(self, request, validate)

        if (
            self.currentvalue and 
            self.isvalid and 
            validate and 
            not self.luhn_mod_ten(self.currentvalue)):
            
            self.isvalid = False
            self.valid_message = 'invalid credit card number'

   
    def luhn_mod_ten(self, ccnumber):
        """ checks to make sure that the card passes a luhn mod-10 checksum.
        
        courtesy: http://aspn.activestate.com
        """

        sum = 0
        num_digits = len(ccnumber)
        oddeven = num_digits & 1

        for count in range(0, num_digits):
            digit = int(ccnumber[count])

            if not (( count & 1 ) ^ oddeven ):
                digit = digit * 2
            if digit > 9:
                digit = digit - 9

            sum = sum + digit

        return ( (sum % 10) == 0 )
    
    
class DateFormField(CompoundFormField):
    def __init__(self, name, fields, yeardeltas = range(-5, 5), required = False, *args, **params):

        elements = {}
        
        for field in fields:
            if field == 'ampm':
                elements[field] = FormField(field, required = required)
            else:
                elements[field] = IntFormField(field, required = required)

        for key in ['year', 'month', 'day']:
            if elements.has_key(key):
                self.hasdate = True
                break
        else:
            self.hasdate = False

        for key in ['hour', 'minute', 'second']:
            if elements.has_key(key):
                self.hastime = True
                break
        else:
            self.hastime = False

            
        assert (self.hasdate or self.hastime)
        
        CompoundFormField.__init__(self, name, elements.values(), **params)

        self.valid_message = ""
        self.required = required
        
        if self.hasdate:
            today = datetime.datetime.today()
            year = today.year
            self.yearrange = [year + i for i in yeardeltas]


    def set_compound_values(self, value):
        if self.hasdate:
            self.elements['year'].set_value(value.year)
            self.elements['month'].set_value(value.month)
            self.elements['day'].set_value(value.day)
            
        if self.hastime:
            if self.elements.has_key('ampm'):
                v = value.hour % 12
                if v == 0: v = 12
                self.elements['hour'].set_value(v)
            else:
                self.elements['hour'].set_value(value.hour)
            self.elements['minute'].set_value(value.minute)
            self.elements['second'].set_value(value.second)
            if self.elements.has_key('ampm'):
                self.elements['ampm'].set_value(value.hour > 12 and 'pm' or 'am')
        
        
    def get_valid_message(self):
        return self.valid_message
        
    def set_request(self, request, validate = True):
        CompoundFormField.set_request(self, request, validate)
        if validate:
            for elem in self.elements.values():
                if elem.is_valid() is False:
                    self.valid_message = 'field "%s": %s' % (self.description, elem.get_valid_message())
                    return

            args = {}

            has_value = False
            if self.hasdate:
                dummy = datetime.date.min
                for key in ['year', 'month', 'day']:
                    if self.elements.has_key(key):
                        args[key] = self.elements[key].currentvalue
                        if args[key] is not None:
                            has_value = True
                    else:
                        args[key] = getattr(dummy, key)
                
            if self.hastime:
                dummy = datetime.time.min
                for key in ['hour', 'minute', 'second']:
                    if self.elements.has_key(key):
                        args[key] = self.elements[key].currentvalue
                        if args[key] is not None:
                            has_value = True
                    else:
                        args[key] = getattr(dummy, key)
                
                if self.elements.has_key('ampm'):
                    if self.elements['ampm'] == 'pm':
                        args['hour'] += 12
                    elif args['hour'] == 12:
                        args['hour'] = 0

            if not has_value:
                self.request_value = None
                return
                
            try:
                if self.hasdate and self.hastime:
                    value = datetime.datetime(**args)
                elif self.hasdate:
                    value = datetime.date(**args)
                else:
                    value = datetime.time(**args)
                self.request_value = value
                self.currentvalue = value
                self.isvalid = True
            except TypeError, e:
                self.isvalid = False
                self.currentvalue = None
                self.valid_message = 'field "%s" does not contain a valid date/time (%s)' % (self.description, str(e))
            except ValueError, e:
                self.isvalid = False
                self.currentvalue = None
                self.valid_message = 'field "%s" does not contain a valid date/time (%s)' % (self.description, str(e))
            
            



# $Id$
# requestbuffer.py - help for dealing with various character encodings
#
# Copyright (C) 2006 Geoffrey T. Dairiki <dairiki@dairiki.org>
#                and Michael Bayer <mike_mp@zzzcomputing.com>
#
# This module is part of Myghty and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php
#
import codecs, sys, StringIO
        
class _Link(object):
    def __init__(self, parent):
        self.parent = parent
    
    def write(self, text):
        """Write text to the buffer chain.

        See also `RequestBuffer.write()`_.
        """
        raise NotImplementedError, "(pure virtual)"
    
    def flush(self):
        pass

    def clear(self):
        pass

    def __repr__(self):
        return "<%s: Parent:\n %s>" % (
            object.__repr__(self)[1:-1], repr(self.parent))

class _Buffer(_Link):
    def __init__(self, parent, buffer=None):
        _Link.__init__(self, parent)
        if buffer is None:
            buffer = StringIO.StringIO()
        self.buffer = buffer
        self.write = self.buffer.write

    def write(self, text):
        # XXX: Note that this method never really gets called.
        #      (See __init__.)
        self.buffer.write(text)
        
    def flush(self):
        self.buffer.seek(0)
        self.parent.write(self.buffer.read())
        self.clear()
        
    def clear(self):
        self.buffer.truncate(0)
        # work around for bug in python2.3's cStringIO:
        try: self.buffer.seek(0)
        except: pass

class _TransparentBuffer(_Buffer):
    """A buffer which defers all recoding.
    """
    def __init__(self, parent):
        _Link.__init__(self, parent)
        self._buf = []

    def write(self, text):
        self._buf.append(text)

    def flush(self):
        self.parent.write(''.join(self._buf))
        self.clear()

    def clear(self):
        self._buf = []

class _CaptureBuffer(_Buffer):
    """A buffer used to capture output.
    """
    pass
    
class _EncodedBuffer(_Buffer):
    """This is a buffer which expects encoded strings.
    """
    def __init__(self, parent, buffer,
                 encoding=sys.getdefaultencoding(), errors='strict'):
        _Buffer.__init__(self, parent, buffer)
        self._writer = None
        self.set_encoding(encoding, errors)
        
    def set_encoding(self, encoding, errors):
        if self._writer:
            self._writer.reset()
        stream_writer = codecs.lookup(encoding)[3]
        self._writer = stream_writer(self.buffer, errors)
        self.write = self._writer.write
        self.__encoding = (encoding, errors)

    def write(self, text):
        self._writer.write(text)
        
    def flush(self):
        self._writer.reset()
        _Buffer.flush(self)
        
    def clear(self):
        self._writer.reset()
        _Buffer.clear(self)

    def get_encoding(self):
        return self.__encoding[0]
    encoding = property(get_encoding,
                        doc="Encoding of the output stream.")
    
    def get_errors(self):
        return self.__encoding[1]
    errors = property(get_errors,
                      doc="Encoding error handling strategy of the output stream.")
    
class _StrBuffer(_Buffer):
    """This is a buffer which strs.
    """
    def __init__(self, parent, buffer):
        _Buffer.__init__(self, parent, buffer)

    def get_encoding(self):
        return None
    encoding = property(get_encoding,
                        doc="Encoding of the output stream.")
    
    def get_errors(self):
        return None
    errors = property(get_errors,
                      doc="Encoding error handling strategy of the output stream.")

class _Filter(_Link):
    def __init__(self, parent, filter):
        _Link.__init__(self, parent)
        self.filter = filter

class _UnicodeFilter(_Filter):
    def write(self, text):
        self.parent.write(unicode(self.filter(text)))

class _StrFilter(_Filter):
    def write(self, text):
        self.parent.write(str(self.filter(text)))

################################################################

class MismatchedPop(Exception):
    pass


class _RequestBuffer(object):
    def __init__(self, top):
        self._top = self.__root = top

    def push_buffer(self):
        '''Push a component output buffer.

        These buffers are used to buffer component output temporarily.
        When the buffer is flushed, it passes it contents on to the next
        buffer in the stack.  Normally the buffers are flushed when
        they are popped off the stack.
        '''
        self.__push(_TransparentBuffer)

    def pop_buffer(self, discard=False):
        """Pop a component output buffer

        The argument ``discard``, if set,
        specifies that the contents of the buffer are to be discarded,
        rather than passed on to the next buffer in the stack.
        """
        buf = self.__pop(_TransparentBuffer)
        if not discard:
            buf.flush()

    def push_filter(self, filter):
        """Push a filter.

        The filter function gets passed either a ``unicode`` or
        a system default encoded ``str``.

        If the ``disable_unicode`` config parameter is set, the filter
        always gets a ``str``.

        The filter can return any type acceptable to the `write`_ method.
        """
        self.__push(self._FilterType, filter=filter)

    def pop_filter(self):
        """Pop a filter.

        Returns the filter function.
        """
        return self.__pop(self._FilterType).filter
    
    def push_capture_buffer(self, buffer=None):
        """Push a capture buffer.

        This is a buffer used to capture component output.
        It does not propagate its contents down the buffer chain
        when it is popped.

        The ``buffer`` should be a file-like object supporting
        at least the ``write()`` method.  The strings passed to
        ``buffer.write()`` will be ``unicode``s or system default
        (usually ASCII) encoded ``str``s.
        """
        self.__push(_CaptureBuffer, buffer=buffer)

    def pop_capture_buffer(self):
        """Pop a capture buffer of the stack.

        Returns the underlying buffer.
        """
        return self.__pop(_CaptureBuffer).buffer

    def __push(self, link_class, **kw):
        self._top = link_class(self._top, **kw)
        return self._top
    
    def __pop(self, *expected_link_classes):
        link = self._top
        if not isinstance(link, expected_link_classes):
            raise MismatchedPop, \
                  "RequestBuffer.pop* expected %s, got %s" \
                  % ( repr(map(lambda x: x.__name__, expected_link_classes)),
                      link.__class__.__name__ )
        self._top = link.parent
        return link

    def get_state(self):
        """Get the current state of the buffer stack.

        Returns an opaque value, which can be pssed to `pop_to_state`_
        to restore the buffer to it's current state.
        """
        return (self._top,)

    def pop_to_state(self, state, discard=False):
        """Pop buffers, and restore saved state.

        Buffers are popped from the stack back to the position
        of the saved state.   (The buffers are flushed as they are
        popped, unless ``discard`` parameter is set.)
        After the stack is unwound, the ``errors``
        attribute is restored to what it was at the time the
        state was saved.

        If a ``UnicodeError`` is thrown while flushing buffers,
        no further buffers are flushed.  However the stack is still
        unwound and the buffer state restored before the exception
        is re-raised.
        """
        top, = state
        if not isinstance(top, _Link):
            raise ValueError, "invalid state"
        
        bufs_to_flush = []

        for b in self.__bufs(end=None):
            if b is top:
                break
            elif not isinstance(b, _CaptureBuffer):
                bufs_to_flush.append(b)
        else:
            # states top is not in our stack
            raise ValueError, "invalid state"

        try:
            if not discard:
                for buf in bufs_to_flush:
                    buf.flush()
        finally:
            self._top = top

    def __bufs(self, begin=None, end="root"):
        if begin is None:
            begin = self._top
        if end == "root":
            end = self.__root
        b = begin
        while b is not end:
            yield b
            b = b.parent
            

    def write(self, text):
        """Write text to the buffer.
        """
        raise NotImplementedError, "pure virtual"

    def flush(self):
        """Flush output.

        Output is flushed up the first capture buffer in the stack.  If
        there is no capture buffer, then output is flushed all the way
        to the final output buffer.
        """
        for buf in self.__bufs():
            if isinstance(buf, _CaptureBuffer):
                break
            buf.flush()

    def clear(self):
        """Clears all buffers in the stack.
        """
        for buf in self.__bufs():
            buf.clear()

    encoding = property(
        lambda self: self.__root.encoding,
        doc="""The current output encoding of the buffer.

        Output written to the underlying output buffer (e.g.
        the Apache request object) are encoded using this encoding.
        """)

    errors = property(
        lambda self: self.__root.errors,
        doc="""The current encoding error handling strategy.

        This is the error handling strategy of the underlying output
        buffer.
        """)

    def set_encoding(self, encoding, errors='strict'):
        """Set encoding of output to the final buffer.
        """
        self.__root.set_encoding(encoding, errors)
        
    def __repr__(self):
        return "<%s: %s>" % (self.__class__.__name__, repr(self._top))

class UnicodeRequestBuffer(_RequestBuffer):

    _FilterType = _UnicodeFilter
    
    def __init__(self, output_buffer, output_encoding, errors='strict'):
        """Constructor.

        The ``output_buffer`` is assumed to be a buffer like stdout
        or an apache request object, so we will not do anything
        but ``.write()`` to it.
        """
        top = _EncodedBuffer(None, output_buffer, output_encoding, errors)
        _RequestBuffer.__init__(self, top)

    def write(self, text):
        """Write text to the buffer.

        .. _`RequestBuffer.write()`:
        
        If ``text`` is a plain ``str``, it is interpreted according
        to the value of the system default encoding
        (obtainable from ``sys.getdefaultencoding()``.)

        Otherwise, if ``text`` is a ``unicode`` object or has a
        ``__unicode__`` method, it is treated as unicode.

        Finally, if ``text`` is neither a ``str`` nor convertable to
        a ``unicode``, it is coerced to a ``str`` and interpreted
        according to the system default encoding.
        """
        if text is None:
            return
        self._top.write(unicode(text))

class StrRequestBuffer(_RequestBuffer):

    _FilterType = _StrFilter
    
    def __init__(self, output_buffer):
        """Constructor.

        The ``output_buffer`` is assumed to be a buffer like stdout
        or an apache request object, so we will not do anything
        but ``.write()`` to it.
        """
        _RequestBuffer.__init__(self, _StrBuffer(None, output_buffer))

    def write(self, text):
        """Write text to the buffer.
        """
        if text is None:
            return
        self._top.write(str(text))




from paste.script.templates import Template

class RoutesTemplate(Template):
    _template_dir = '../paster_templates/routes'
    summary = 'Routes Template'
    
class SimpleTemplate(Template):
    _template_dir = '../paster_templates/simple'
    summary = 'Simple Template'

class MCTemplate(Template):
    _template_dir = '../paster_templates/modulecomponents'
    summary = 'Module Component Template'
    



from zblog.controller import *
import re
from zblog.domain.blog import *
import zblog.domain.actions as actions
import zblog.util.form as formutil
import zblog.database.mappers as mapper

class Comments(Controller):
    @access_control()
    def __call__(self, m, ARGS):
        """produces a listing of comments for a post"""
        match = re.search(r'/(\w+)/$', m.request_path)
        if match:
            post_id = match.group(1)
        else:
            m.abort(404)

        post = Post.mapper.get(post_id)
        if post is None:
            m.abort(404)

        form = self.commentform(m, ARGS, post)
        self.template(m, '/blog/comments.myt', post=post, form=form)

    @access_control()
    def view(self, m, ARGS):
        """views a specific comment"""
        match = re.search(r'/(\w+)/$', m.request_path)
        if match:
            comment_id = match.group(1)
        else:
            m.abort(404)

        comment = Comment.mapper.get(comment_id)
        if comment is None:
            m.abort(404)

        form = self.commentform(m, ARGS, comment.post, comment)
        self.template(m, '/blog/postcomment.myt', post=comment.post, comment=comment, form=form)
        
    @access_control(action=actions.CreateComment())
    def post(self, m, ARGS, post_id, parent_comment_id, confirm=False, preview=False):
        """posts a comment, or provides a preview display."""
        post = Post.mapper.get(post_id)
        if post is None:
            m.abort(404)

        if parent_comment_id:
            parentcomment = Comment.mapper.get(parent_comment_id)
        else:
            parentcomment = None
        form = self.commentform(m, ARGS, post, comment=parentcomment)
        form.set_request(ARGS, validate=True)
        if not form.is_valid():
            self.template(m, '/blog/postcomment.myt', post=post, form=form, comment=parentcomment)
            return
        if int(preview):
            comment = self.createcomment(m, post, form, parentcomment)
            self.template(m, '/blog/postcomment.myt', post=post, form=form, comment=parentcomment, preview=comment)
            return
        mapper.begin()
        comment = self.createcomment(m, post, form, parentcomment)
        mapper.commit()
        form = self.commentform(m, ARGS, post, comment)
        form.append_success("Comment posted!")
        self.template(m, '/blog/postcomment.myt', post=post, comment=comment, form=form)

    def createcomment(self, m, post, form, parentcomment):
        """creates a new comment from a given form"""
        comment = Comment() 
        comment.id = None
        form.reflect_to(comment)
        comment.user = self.get_user(m)
        comment.post = post
        if parentcomment is not None:
            comment.parent = parentcomment
        return comment
        
    def commentform(self, m, ARGS, post, comment=None):
        """creates a blank comment form"""
        if comment is not None:
            subject = "Re: %s" % comment.subject
        else:
            subject = None

        form=formutil.Form('comment', [
            formutil.IntFormField('post_id'),
            formutil.IntFormField('preview'),
            formutil.IntFormField('parent_comment_id'),
            formutil.FormField('subject', required=True, default=subject),
            formutil.FormField('body', required=True)
        ])
        form['post_id'].value = post.id
        if comment is not None:
            form['parent_comment_id'].value = comment.id
            
        return form
        
index = Comments()    

        
